@@ -21,27 +21,30 @@
 
 static bool IsActive;
 
-#define MK_COLOR_A(r,g,b,a) ( ((a)<<surface->format->Ashift) | ((r)<<surface->format->Rshift) | ((g) << surface->format->Gshift) | ((b) << surface->format->Bshift))
-
-void Help_Draw(SDL_Surface *surface, const SDL_Rect *rect)
+void Help_Draw(MDFN_Surface *surface, const MDFN_Rect *rect)
 {
  if(!IsActive) return;
 
- uint32 * pixels = (uint32 *)surface->pixels;
- uint32 pitch32 = surface->pitch >> 2;
+ uint32 bgcolor = surface->MakeColor(0, 0, 0, 0xFF);
+ uint32 *pixels = surface->pixels;
+ uint32 pitch32 = surface->pitch32;
 
- for(unsigned int y = 0; y < rect->h; y++)
+ for(int y = 0; y < rect->h; y++)
  {
   uint32 *row = pixels + y * pitch32;
-  for(unsigned int x = 0; x < rect->w; x++)
-   row[x] = MK_COLOR_A(0, 0, 0, 0xFF); //0xE0);
+
+  for(int x = 0; x < rect->w; x++)
+   row[x] = bgcolor;
  }
- static const char *HelpStrings[][2] = { 
+
+ static const char *HelpStrings[][2] = {
 	{ "F1", gettext_noop("Toggle Help") },
 	{ "-", NULL },
 	{ "F2", gettext_noop("Configure Command Key") },
+	{ "SHIFT + F2", gettext_noop("Configure Command Key, for logical AND mode") },
 	{ "ALT + SHIFT + [n]", gettext_noop("Configure buttons on port n(1-5)") },
 	{ "CTRL + SHIFT + [n]", gettext_noop("Select device on port n") },
+	{ "Scroll Lock", gettext_noop("Toggle input grabbing(for emulated mice)") },
 	{ "-", NULL },
 	{ "ALT + O", gettext_noop("Rotate Screen") },
 	{ "ALT + ENTER", gettext_noop("Toggle Fullscreen Mode") },
@@ -50,8 +53,8 @@ void Help_Draw(SDL_Surface *surface, const SDL_Rect *rect)
 	{ "ALT + S", gettext_noop("Enable state rewinding") },
 	{ "BACKSPACE", gettext_noop("Rewind") },
 	{ "F5", gettext_noop("Save State") },
-	{ "F6", gettext_noop("Select Disk") },
 	{ "F7", gettext_noop("Load State") },
+        { "F6", gettext_noop("Select Disk") },
 	{ "F8", gettext_noop("Insert coin; Insert/Eject disk") },
 	{ "-", NULL },
 	{ "F10", gettext_noop("(Soft) Reset, if available on emulated system.") },
@@ -61,36 +64,33 @@ void Help_Draw(SDL_Surface *surface, const SDL_Rect *rect)
 
  unsigned y = 0;
 
- DrawTextTrans(pixels + pitch32 * y, surface->pitch, rect->w, (UTF8*)_("Mednafen Quickie Help"), MK_COLOR_A(0x00,0xFF,0x00,0xFF), TRUE, MDFN_FONT_9x18_18x18);
- y += 30;
-
- DrawTextTrans(pixels + pitch32 * y, surface->pitch, rect->w, (UTF8 *)_("Default key assignments:"), MK_COLOR_A(0xC0,0xC0,0xC0,0xFF), false, MDFN_FONT_9x18_18x18);
+ DrawTextTrans(pixels + pitch32 * y, surface->pitch32 << 2, rect->w, (UTF8 *)_("Default key assignments:"), surface->MakeColor(0x00,0xFF,0x00,0xFF), false, MDFN_FONT_9x18_18x18);
  y += 18;
 
  for(unsigned int i = 0; i < sizeof(HelpStrings) / sizeof(HelpStrings[0]); i++)
  {
-  if((y + 18) > rect->h)
+  if((int)(y + 18) > rect->h)
    break;
 
   unsigned x = 0;
   if(HelpStrings[i][0][0] == '-')
   {
    y -= 4;
-   DrawTextTrans(pixels + pitch32 * y, surface->pitch, rect->w, (UTF8*)" -------------------------------------------------------", MK_COLOR_A(0x60,0x60,0x60,0xFF), FALSE, MDFN_FONT_9x18_18x18);
+   DrawTextTrans(pixels + pitch32 * y, surface->pitch32 << 2, rect->w, (UTF8*)" -------------------------------------------------------", surface->MakeColor(0x60,0x60,0x60,0xFF), FALSE, MDFN_FONT_9x18_18x18);
    y += 14;
   }
   else
   {
    x += 9;
 
-   x += DrawTextTrans(pixels + pitch32 * y + x, surface->pitch, rect->w - x, (UTF8*)HelpStrings[i][0],
-	MK_COLOR_A(0x40,0xDF,0x40,0xFF), FALSE, MDFN_FONT_9x18_18x18);
+   x += DrawTextTrans(pixels + pitch32 * y + x, surface->pitch32 << 2, rect->w - x, (UTF8*)HelpStrings[i][0],
+	surface->MakeColor(0x40,0xDF,0x40,0xFF), FALSE, MDFN_FONT_9x18_18x18);
 
-   x += DrawTextTrans(pixels + pitch32 * y + x, surface->pitch, rect->w - x, (UTF8*)" - ",
-        MK_COLOR_A(0x40,0x40,0x40,0xFF), FALSE, MDFN_FONT_9x18_18x18);
+   x += DrawTextTrans(pixels + pitch32 * y + x, surface->pitch32 << 2, rect->w - x, (UTF8*)" - ",
+        surface->MakeColor(0x40,0x40,0x40,0xFF), FALSE, MDFN_FONT_9x18_18x18);
 
-   x += DrawTextTrans(pixels + pitch32 * y + x, surface->pitch, rect->w - x, (UTF8*)_(HelpStrings[i][1]),
-        MK_COLOR_A(0xC0,0xC0,0xC0,0xFF), FALSE, MDFN_FONT_9x18_18x18);
+   x += DrawTextTrans(pixels + pitch32 * y + x, surface->pitch32 << 2, rect->w - x, (UTF8*)_(HelpStrings[i][1]),
+        surface->MakeColor(0xC0,0xC0,0xC0,0xFF), FALSE, MDFN_FONT_9x18_18x18);
 
    y += 18;
   }
