@@ -22,7 +22,42 @@
 #include "input.h"
 #include "input-config.h"
 #include "video.h"
-#include "joystick.h"
+#include "Joystick.h"
+
+extern JoystickManager *joy_manager;
+
+int32 DTestMouseAxis(ButtConfig &bc, const char* KeyState, const uint32* MouseData, const bool axis_hint)	// UNDOCUMENTED INCOMPLETE FUN
+{
+ if(bc.ButtType == BUTTC_JOYSTICK)	// HAXY
+ {
+  int32 ret = 0;
+  ButtConfig tmp_bc = bc;
+
+  tmp_bc.ButtonNum ^= 0x4000;
+
+  ret = joy_manager->TestAnalogButton(bc);
+  ret -= joy_manager->TestAnalogButton(tmp_bc);
+  ret += 32768;
+
+  ret = PtoV_J(ret, axis_hint, (bool)(tmp_bc.ButtonNum & (1 << 18)));
+
+  return(ret);
+ }
+ else if(bc.ButtType == BUTTC_MOUSE)
+ {
+  if(bc.DeviceNum == 0)
+  {
+   if(bc.ButtonNum & 0x8000)
+    return(MouseData[bc.ButtonNum & 1]);
+   else
+    return(0);
+  }
+  else
+   return(0);
+ }
+ else					// UNSUPPORTED
+  return(0);
+}
 
 static bool DTestButtonMouse(const ButtConfig &bc, const uint32 *MouseData)
 {
@@ -57,7 +92,7 @@ static bool DTestButtonMouse(const ButtConfig &bc, const uint32 *MouseData)
  }
  else 
  {
-  if(MouseData[2] & (1 << SDL_BUTTON(bc.ButtonNum)))
+  if(MouseData[2] & (1U << bc.ButtonNum))
    return(true);
  }
  return(false);
@@ -66,27 +101,33 @@ static bool DTestButtonMouse(const ButtConfig &bc, const uint32 *MouseData)
 int DTestButton(std::vector<ButtConfig> &bc, const char *KeyState, const uint32 *MouseData, bool analog)
 {
  const int maxv = analog ? 32767 : 1;
+ int32 ret = 0;	// Will obviously break if 10s of thousands of physical buttons are assigned to one emulated analog button. ;)
 
  for(unsigned int x = 0; x < bc.size(); x++)
  {
-  if(bc[x].ButtType==BUTTC_KEYBOARD)
+  if(bc[x].ButtType == BUTTC_KEYBOARD)
   {
    if(KeyState[bc[x].ButtonNum])
-    return(maxv);
+    ret += maxv;
   }
-  else if(bc[x].ButtType==BUTTC_JOYSTICK)
+  else if(bc[x].ButtType == BUTTC_JOYSTICK)
   {
    if(analog)
-    return DTestAnalogButtonJoy(bc[x]);
-   if(DTestButtonJoy(bc[x]))
-    return(maxv);
+    ret += joy_manager->TestAnalogButton(bc[x]);
+   else if(joy_manager->TestButton(bc[x]))
+    ret += maxv;
   }
   else if(bc[x].ButtType == BUTTC_MOUSE)
   {
-   return(DTestButtonMouse(bc[x], MouseData) ? maxv : 0);
+   if(DTestButtonMouse(bc[x], MouseData))
+    ret += maxv;
   }
  }
- return(0);
+
+ if(ret > maxv)
+  ret = maxv;
+
+ return(ret);
 }
 
 int DTestButton(ButtConfig &bc, const char *KeyState, const uint32 *MouseData, bool analog)
@@ -103,10 +144,11 @@ int DTestButton(ButtConfig &bc, const char *KeyState, const uint32 *MouseData, b
 #define ICSS_CTRL	4
 
 /* Used for command keys */
-int DTestButtonCombo(std::vector<ButtConfig> &bc, const char *KeyState, const uint32 *MouseData)
+int DTestButtonCombo(std::vector<ButtConfig> &bc, const char *KeyState, const uint32 *MouseData, bool AND_Mode)
 {
  unsigned int x;
  unsigned int ss = 0;
+ unsigned match_count = 0;
 
  if(KeyState[MKK(LALT)] || KeyState[MKK(RALT)]) ss |= ICSS_ALT;
  if(KeyState[MKK(LSHIFT)] || KeyState[MKK(RSHIFT)]) ss |= ICSS_SHIFT;
@@ -117,37 +159,44 @@ int DTestButtonCombo(std::vector<ButtConfig> &bc, const char *KeyState, const ui
   if(bc[x].ButtType == BUTTC_KEYBOARD)
   {
    uint32 b = bc[x].ButtonNum;
-   
+
    if(KeyState[b & 0xFFFF] && ((b >> 24) == ss))
-    return(1);
+    match_count++;
   }
   else if(bc[x].ButtType == BUTTC_JOYSTICK)
   {
-   if(DTestButtonJoy(bc[x]))
-    return(1);
+   if(joy_manager->TestButton(bc[x]))
+    match_count++;
   }
   else if(bc[x].ButtType == BUTTC_MOUSE)
   {
-   return(DTestButtonMouse(bc[x], MouseData) ? 1 : 0);
+   if(DTestButtonMouse(bc[x], MouseData))
+    match_count++;
   }
  }
+
+ if(match_count > 0)
+ {
+  if(!AND_Mode || match_count == bc.size())
+   return(1);
+ }
+
  return(0);
 }
 
-int DTestButtonCombo(ButtConfig &bc, const char *KeyState, const uint32 *MouseData)
+int DTestButtonCombo(ButtConfig &bc, const char *KeyState, const uint32 *MouseData, bool AND_Mode)
 {
  std::vector<ButtConfig> neobc;
  neobc.push_back(bc);
 
- return(DTestButtonCombo(neobc, KeyState, MouseData));
+ return(DTestButtonCombo(neobc, KeyState, MouseData, AND_Mode));
 }
 
 
 static ButtConfig efbc;
 static int volatile efbcdone;
 static int volatile efck;
-static unsigned int JoyFilter;
-static int32 LastAx[64][64];
+
 static int LastMouseX;
 static int LastMouseY;
 
@@ -159,13 +208,20 @@ static int EventFilter(const SDL_Event *event)
  switch(event->type)
  {
    case SDL_KEYDOWN:    if(!efck || (event->key.keysym.sym != MKK(LALT) && event->key.keysym.sym != MKK(RALT) &&
-                         event->key.keysym.sym != MKK(LSHIFT) && event->key.keysym.sym != MKK(RSHIFT)))
+                         event->key.keysym.sym != MKK(LSHIFT) && event->key.keysym.sym != MKK(RSHIFT) &&
+			 event->key.keysym.sym != MKK(LCTRL) && event->key.keysym.sym != MKK(RCTRL)))
                         {
                                 efbc.ButtType = BUTTC_KEYBOARD;
                                 efbc.DeviceNum = 0;
                                 efbc.ButtonNum = event->key.keysym.sym;
+
+				if(0 == event->key.keysym.sym)
+				 printf("*** NULL KEYSYM! ***\n");
+
+				//printf("%u\n", event->key.keysym.sym);
+
                                 if(efck)
-                                        efbc.ButtonNum |= ((event->key.keysym.mod & KMOD_ALT) ? (ICSS_ALT<<24):0) | ((event->key.keysym.mod & KMOD_SHIFT) ? (ICSS_SHIFT<<24):0);
+                                        efbc.ButtonNum |= ((event->key.keysym.mod & KMOD_ALT) ? (ICSS_ALT<<24):0) | ((event->key.keysym.mod & KMOD_SHIFT) ? (ICSS_SHIFT<<24):0) | ((event->key.keysym.mod & KMOD_CTRL) ? (ICSS_CTRL<<24):0);
 				efbcdone = 1;
                                 return(1);
                         }
@@ -177,72 +233,14 @@ static int EventFilter(const SDL_Event *event)
   		   	    efbcdone = 1;
 			    return(1);
 
-  case SDL_JOYBUTTONDOWN: 
-			  if(JoyFilter & ICJF_DIGITAL_BUTTON)
-			  {
-			   efbc.ButtType = BUTTC_JOYSTICK;
-                           efbc.DeviceNum = event->jbutton.which;
-                           efbc.ButtonNum = event->jbutton.button;
-			   efbc.DeviceID = GetJoystickUniqueID(event->jbutton.which);
-			   efbcdone = 1;
-                           return(1);
-			  }
-			  break;
-
-   case SDL_JOYHATMOTION:                         
-			//printf("Raw: %d\n", SDL_JoystickGetHat(Joysticks[event->jhat.which], 0xFF)); //(bc->ButtonNum[x]>>8)&0x1F) & (bc->ButtonNum[x]&0xFF)));
-			if(JoyFilter & ICJF_HAT)
-			{
-			 if(event->jhat.value != SDL_HAT_CENTERED && event->jhat.value != SDL_HAT_RIGHTUP && event->jhat.value != SDL_HAT_RIGHTDOWN &&
-				event->jhat.value != SDL_HAT_LEFTUP && event->jhat.value != SDL_HAT_LEFTDOWN)
-                         {
-				//printf("Real: %d\n",event->jhat.value);
-				//break;
-                          efbc.ButtType = BUTTC_JOYSTICK;
-                          efbc.DeviceNum = event->jhat.which;
-                          efbc.ButtonNum = 0x2000|((event->jhat.hat&0x1F)<<8)|event->jhat.value;
-			  efbc.DeviceID = GetJoystickUniqueID(event->jhat.which);
-			  efbcdone = 1;
-                          return(1);
-                         }
-			}
-                        break;
-
-   case SDL_JOYAXISMOTION:
-	//printf("%d %d %d\n", event->jaxis.which, event->jaxis.axis, event->jaxis.value);
-	if(JoyFilter & ICJF_AXIS)
-	{
-         if(LastAx[event->jaxis.which][event->jaxis.axis]==0x100000)
-         {
-          if(abs(event->jaxis.value)<1000)
-           LastAx[event->jaxis.which][event->jaxis.axis]=event->jaxis.value;
-         }
-         else
-         {
-          if(abs(LastAx[event->jaxis.which][event->jaxis.axis]-event->jaxis.value)>=24000)
-          {
-           efbc.ButtType = BUTTC_JOYSTICK;
-           efbc.DeviceNum = event->jaxis.which;
-           efbc.ButtonNum = 0x8000|(event->jaxis.axis)|((event->jaxis.value<0)?0x4000:0);
-	   efbc.DeviceID = GetJoystickUniqueID(event->jaxis.which);
-	   efbcdone = 1;
-
-	   //puts("axis");
-	   return(1);
-          }
-         }
-	}
-        break;
  }
 
  return(1);
 }
 
 
-int DTryButtonBegin(ButtConfig *bc, int commandkey, unsigned int jf)
+int DTryButtonBegin(ButtConfig *bc, int commandkey)
 {
- JoyFilter = jf;
-
  memcpy(&efbc, bc, sizeof(ButtConfig));
  efck = commandkey;
  efbcdone = 0;
