@@ -98,18 +98,12 @@ static const ButtConfig WSwanPadConfig[12] =
 };
 
 
-static const ButtConfig powerpadsc[2][12]={
-                              {
-                                MK(o),MK(p),MK(LEFTBRACKET),MK(RIGHTBRACKET),
-                                MK(k),MK(l),MK(SEMICOLON),MK(QUOTE),
-                                MK(m),MK(COMMA),MK(PERIOD),MK(SLASH)
-                              },
-                              {
-                                MK(o),MK(p),MK(LEFTBRACKET),MK(RIGHTBRACKET),
-                                MK(k),MK(l),MK(SEMICOLON),MK(QUOTE),
-                                MK(m),MK(COMMA),MK(PERIOD),MK(SLASH)
-                              }
-                             };
+static const ButtConfig PowerPadConfig[12] =
+{
+ MK(o),MK(p),MK(LEFTBRACKET),MK(RIGHTBRACKET),
+ MK(k),MK(l),MK(SEMICOLON),MK(QUOTE),
+ MK(m),MK(COMMA),MK(PERIOD),MK(SLASH)
+};
 
 static const ButtConfig fkbmap[0x48]=
 {
@@ -146,18 +140,36 @@ static const ButtConfig FTrainerButtons[12]=
                                MK(m),MK(COMMA),MK(PERIOD),MK(SLASH)
 };
 
-static ButtConfig OekaKidsConfig[1] =
+static ButtConfig OekaKidsConfig[3] =
 {
+ { BUTTC_MOUSE, 0, 0x8000, 0 },
+ { BUTTC_MOUSE, 0, 0x8001, 0 },
+
  { BUTTC_MOUSE, 0, 0, 0 },
 };
 
-static ButtConfig ArkanoidConfig[1] =
+static ButtConfig ArkanoidConfig[2] =
 {
+ { BUTTC_MOUSE, 0, 0x8000, 0 },
+
  { BUTTC_MOUSE, 0, 0, 0 },
 };
 
-static ButtConfig NESZapperConfig[2] =
+static ButtConfig ShadowConfig[4] =
 {
+ { BUTTC_MOUSE, 0, 0x8000, 0 },
+ { BUTTC_MOUSE, 0, 0x8001, 0 },
+
+ { BUTTC_MOUSE, 0, 0, 0 },
+ { BUTTC_MOUSE, 0, 2, 0 },
+};
+
+
+static ButtConfig NESZapperConfig[4] =
+{
+ { BUTTC_MOUSE, 0, 0x8000, 0 },
+ { BUTTC_MOUSE, 0, 0x8001, 0 },
+
  { BUTTC_MOUSE, 0, 0, 0 },
  { BUTTC_MOUSE, 0, 2, 0 },
 };
@@ -364,11 +376,46 @@ static const ButtConfig CDPlayInputConfig[] =
 
 static const ButtConfig MDPad3Config[2][8] =
 {
- { MK(w), MK(s), MK(a), MK(d), MK(KP1), MK(KP2), MK(KP3), MK(RETURN) },
+ { MK(w), MK(s), MK(a), MK(d), MK(KP2), MK(KP3), MK(KP1), MK(RETURN) },
  { MKZ(), MKZ(), MKZ(), MKZ(), MKZ(),   MKZ(),   MKZ(),   MKZ() },
 
 };
 
+static const ButtConfig SNESPadConfig[12] =
+{
+ MK(KP2),
+ MK(KP4),
+ MK(TAB),
+ MK(RETURN),
+ MK(w),
+ MK(s),
+ MK(a),
+ MK(d),
+ MK(KP6),
+ MK(KP8),
+ MK(KP7),
+ MK(KP9),
+};
+
+
+static ButtConfig SNESMouseConfig[2] =
+{
+ { BUTTC_MOUSE, 0, 0, 0 },
+ { BUTTC_MOUSE, 0, 2, 0 },
+};
+
+static ButtConfig SNESSuperScopeConfig[7] =
+{
+ { BUTTC_MOUSE, 0, 0x8000, 0 },
+ { BUTTC_MOUSE, 0, 0x8001, 0 },
+
+ { BUTTC_MOUSE, 0, 0, 0 },	// Trigger
+ MK(SPACE),			// Away trigger
+ { BUTTC_MOUSE, 0, 1, 0 },	// Pause
+ MK(END),			// Turbo
+ { BUTTC_MOUSE, 0, 2, 0 },	// Cursor
+};
+
 /*
  { "select", "SELECT", 4, IDIT_BUTTON, NULL },
  { NULL, "empty", 0, IDIT_BUTTON },
@@ -409,12 +456,63 @@ static const ButtConfig PSXPadConfig[14] =
  MK(KP4),
 };
 
+static const ButtConfig PSXDancePadConfig[10] =
+{
+ MK(KP_DIVIDE),
+ MK(KP_MULTIPLY),
+ MK(KP8),
+ MK(KP6),
+ MK(KP2),
+ MK(KP4),
+
+ MK(KP1),
+ MK(KP9),
+ MK(KP7),
+ MK(KP3),
+};
+
 static ButtConfig PSXMouseConfig[2] =
 {
  { BUTTC_MOUSE, 0, 2, 0 },
  { BUTTC_MOUSE, 0, 0, 0 },
 };
 
+static ButtConfig PSXGunConConfig[6] =
+{
+ { BUTTC_MOUSE, 0, 0x8000, 0 },
+ { BUTTC_MOUSE, 0, 0x8001, 0 },
+
+ { BUTTC_MOUSE, 0, 0, 0 },
+ { BUTTC_MOUSE, 0, 2, 0 },
+ { BUTTC_MOUSE, 0, 1, 0 },
+ MK(SPACE)
+};
+
+static ButtConfig PSXJustifierConfig[6] =
+{
+ { BUTTC_MOUSE, 0, 0x8000, 0 },
+ { BUTTC_MOUSE, 0, 0x8001, 0 },
+
+ { BUTTC_MOUSE, 0, 0, 0 },
+ { BUTTC_MOUSE, 0, 2, 0 },
+ { BUTTC_MOUSE, 0, 1, 0 },
+ MK(SPACE)
+};
+
+#if 0
+static ButtConfig VBPadConfig[14] =
+{
+
+
+ MK(I),	// RPad, Up
+ MK(L), // RPad, Right
+
+ MK(F), // LPad, Right
+ MK(S), // LPad, Left
+ MK(D), // LPad, Down
+ MK(E), // LPad, Up
+};
+#endif
 
 typedef struct
 {
@@ -430,11 +528,10 @@ static DefaultSettingsMeow defset[] =
  { "nes.input.port3.gamepad", NESGamePadConfig[2], sizeof(NESGamePadConfig[2]) / sizeof(ButtConfig) },
  { "nes.input.port4.gamepad", NESGamePadConfig[3], sizeof(NESGamePadConfig[3]) / sizeof(ButtConfig) },
 
-
- { "nes.input.port1.powerpada", powerpadsc[0], sizeof(powerpadsc[0]) / sizeof(ButtConfig) },
- { "nes.input.port2.powerpada", powerpadsc[1], sizeof(powerpadsc[1]) / sizeof(ButtConfig) },
- { "nes.input.port3.powerpadb", powerpadsc[0], sizeof(powerpadsc[0]) / sizeof(ButtConfig) },
- { "nes.input.port4.powerpadb", powerpadsc[1], sizeof(powerpadsc[1]) / sizeof(ButtConfig) },
+ { "nes.input.port1.powerpada", PowerPadConfig, sizeof(PowerPadConfig) / sizeof(ButtConfig) },
+ { "nes.input.port2.powerpada", PowerPadConfig, sizeof(PowerPadConfig) / sizeof(ButtConfig) },
+ { "nes.input.port1.powerpadb", PowerPadConfig, sizeof(PowerPadConfig) / sizeof(ButtConfig) },
+ { "nes.input.port2.powerpadb", PowerPadConfig, sizeof(PowerPadConfig) / sizeof(ButtConfig) },
 
  { "nes.input.port1.zapper", NESZapperConfig, sizeof(NESZapperConfig) / sizeof(ButtConfig) },
  { "nes.input.port2.zapper", NESZapperConfig, sizeof(NESZapperConfig) / sizeof(ButtConfig) },
@@ -449,6 +546,8 @@ static DefaultSettingsMeow defset[] =
 
  { "nes.input.fcexp.oekakids", OekaKidsConfig, sizeof(OekaKidsConfig) / sizeof(ButtConfig) },
 
+ { "nes.input.fcexp.shadow", ShadowConfig, sizeof(ShadowConfig) / sizeof(ButtConfig) },
+
  { "nes.input.port1.arkanoid", ArkanoidConfig, sizeof(ArkanoidConfig) / sizeof(ArkanoidConfig) },
  { "nes.input.port2.arkanoid", ArkanoidConfig, sizeof(ArkanoidConfig) / sizeof(ArkanoidConfig) },
  { "nes.input.port3.arkanoid", ArkanoidConfig, sizeof(ArkanoidConfig) / sizeof(ArkanoidConfig) },
@@ -501,9 +600,15 @@ static DefaultSettingsMeow defset[] =
  { "md.input.port7.gamepad", MDPad3Config[1], sizeof(MDPad3Config[1]) / sizeof(ButtConfig) },
  { "md.input.port8.gamepad", MDPad3Config[1], sizeof(MDPad3Config[1]) / sizeof(ButtConfig) },
 
+ { "snes.input.port1.gamepad", SNESPadConfig, sizeof(SNESPadConfig) / sizeof(ButtConfig) },
+ { "snes.input.port1.mouse", SNESMouseConfig, sizeof(SNESMouseConfig) / sizeof(ButtConfig) },
+ { "snes.input.port2.mouse", SNESMouseConfig, sizeof(SNESMouseConfig) / sizeof(ButtConfig) },
+ { "snes.input.port2.superscope", SNESSuperScopeConfig, sizeof(SNESSuperScopeConfig) / sizeof(ButtConfig) },
 
  { "psx.input.port1.gamepad", PSXPadConfig, sizeof(PSXPadConfig) / sizeof(ButtConfig) },
 
+ { "psx.input.port1.dancepad", PSXDancePadConfig, sizeof(PSXDancePadConfig) / sizeof(ButtConfig) },
+
  { "psx.input.port1.mouse", PSXMouseConfig, sizeof(PSXMouseConfig) / sizeof(ButtConfig) },
  { "psx.input.port2.mouse", PSXMouseConfig, sizeof(PSXMouseConfig) / sizeof(ButtConfig) },
  { "psx.input.port3.mouse", PSXMouseConfig, sizeof(PSXMouseConfig) / sizeof(ButtConfig) },
@@ -513,6 +618,20 @@ static DefaultSettingsMeow defset[] =
  { "psx.input.port7.mouse", PSXMouseConfig, sizeof(PSXMouseConfig) / sizeof(ButtConfig) },
  { "psx.input.port8.mouse", PSXMouseConfig, sizeof(PSXMouseConfig) / sizeof(ButtConfig) },
 
+ { "psx.input.port1.guncon", PSXGunConConfig, sizeof(PSXGunConConfig) / sizeof(ButtConfig) },
+ { "psx.input.port2.guncon", PSXGunConConfig, sizeof(PSXGunConConfig) / sizeof(ButtConfig) },
+ { "psx.input.port3.guncon", PSXGunConConfig, sizeof(PSXGunConConfig) / sizeof(ButtConfig) },
+ { "psx.input.port4.guncon", PSXGunConConfig, sizeof(PSXGunConConfig) / sizeof(ButtConfig) },
+ { "psx.input.port5.guncon", PSXGunConConfig, sizeof(PSXGunConConfig) / sizeof(ButtConfig) },
+ { "psx.input.port6.guncon", PSXGunConConfig, sizeof(PSXGunConConfig) / sizeof(ButtConfig) },
+ { "psx.input.port7.guncon", PSXGunConConfig, sizeof(PSXGunConConfig) / sizeof(ButtConfig) },
+ { "psx.input.port8.guncon", PSXGunConConfig, sizeof(PSXGunConConfig) / sizeof(ButtConfig) },
+
+ { "psx.input.port1.justifier", PSXJustifierConfig, sizeof(PSXJustifierConfig) / sizeof(ButtConfig) },
+ { "psx.input.port2.justifier", PSXJustifierConfig, sizeof(PSXJustifierConfig) / sizeof(ButtConfig) },
+
+ //{ "vb.input.builtin.gamepad", VBPadConfig, sizeof(VBPadConfig) / sizeof(VBPadConfig) },
+
  { "mmplay.input.builtin.controller", MMPlayInputConfig, sizeof(MMPlayInputConfig) / sizeof(ButtConfig) },
  { "cdplay.input.builtin.controller", CDPlayInputConfig, sizeof(CDPlayInputConfig) / sizeof(ButtConfig) },
 
