@@ -33,14 +33,12 @@
 #define CEVT_WANT_EXIT		0x80 // Emulator exit or GUI exit or bust!
 
 
-#define CEVT_NP_TEXT_TO_SERVER	0x100
 #define CEVT_NP_DISPLAY_TEXT	0x101
-#define CEVT_NP_TOGGLE_VIEWABLE	0x102
 #define CEVT_NP_TOGGLE_TT	0x103
-#define CEVT_NP_CONNECT		0x104
-#define CEVT_NP_SETNICK		0x105
-#define CEVT_NP_PING		0x106
-#define CEVT_NP_INTEGRITY	0x107
+#define CEVT_NP_LINE            0x180
+#define CEVT_NP_LINE_RESPONSE   0x181
+
+#define CEVT_SET_INPUT_FOCUS	0x1000	// Main thread to game thread.
 
 void SendCEvent(unsigned int code, void *data1, void *data2);
 
@@ -49,34 +47,30 @@ void PauseGameLoop(bool p);
 void SDL_MDFN_ShowCursor(int toggle);
 
 extern int NoWaiting;
+extern bool MDFNDHaveFocus;
 
 extern MDFNGI *CurGame;
 int CloseGame(void);
 
 void RefreshThrottleFPS(double);
 void PumpWrap(void);
-void SetJoyReadMode(int mode);
-void MainSetEventHook(int (*eh)(const SDL_Event *event));
-
 void MainRequestExit(void);
 
-extern bool pending_save_state, pending_snapshot, pending_save_movie;
+extern bool pending_save_state, pending_ssnapshot, pending_snapshot, pending_save_movie;
 
 void DoRunNormal(void);
 void DoFrameAdvance(void);
+bool IsInFrameAdvance(void);
 
-
-void LockGameMutex(bool lock);
 void DebuggerFudge(void);
 
 extern volatile int GameThreadRun;
 
 extern int sdlhaveogl;
 
-bool MT_FromRemote_VideoSync(void);
-bool MT_FromRemote_SoundSync(void);
 
 void GT_ToggleFS(void);
-void GT_ReinitVideo(void);
+bool GT_ReinitVideo(void);
+bool GT_ReinitSound(void);
 
 #endif
