@@ -1,5 +1,5 @@
 int MDFND_NetworkConnect(void);
-void DrawNetplayTextBuffer(SDL_Surface *surface, const SDL_Rect *src_rect);
+void DrawNetplayTextBuffer(MDFN_Surface *surface, const MDFN_Rect *src_rect);
 void NetplayText_InMainThread(uint8 *text, bool NetEcho);
 
 int NetplayEventHook(const SDL_Event *event);
