@@ -1,7 +1,4 @@
 /* Mednafen - Multi-system Emulator
- *
- * Copyright notice for this file:
- *  Copyright (C) 2002 Xodnizel
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,45 +28,47 @@ static SexyAL *Interface = NULL;
 static SexyAL_device *Output = NULL;
 static SexyAL_format format;
 static SexyAL_buffering buffering;
-static SexyAL_enumtype *DriverTypes = NULL;
-static int CurDriverIndex = 0;
 
 static int16 *EmuModBuffer = NULL;
 static int32 EmuModBufferSize = 0;	// In frames.
 
 static double SoundRate = 0;
+static bool NeedReInit = false;
 
-double GetSoundRate(void)
+bool Sound_NeedReInit(void)
+{
+ return NeedReInit;
+}
+
+double Sound_GetRate(void)
 {
  return(SoundRate);
 }
 
-uint32 GetMaxSound(void)
+uint32 Sound_CanWrite(void)
 {
  if(!Output)
   return(0);
 
- return(buffering.buffer_size);
-}
-
-uint32 GetWriteSound(void)
-{
- if(!Output) return(0);
  return(Output->CanWrite(Output));
 }
 
-void WriteSound(int16 *Buffer, int Count)
+void Sound_Write(int16 *Buffer, int Count)
 {
  if(!Output)
   return;
 
  if(!Output->Write(Output, Buffer, Count))
  {
+  //
+  // TODO; We need to take assert()'s out of the wasapi and wasapish code before we can safely enable this
+  //
+  //NeedReInit = true;
   //printf("Output->Write failure? %d\n", Count);
  }
 }
 
-void WriteSoundSilence(int ms)
+void Sound_WriteSilence(int ms)
 {
  unsigned int frames = (uint64)format.rate * ms / 1000;
  int16 SBuffer[frames * format.channels];
@@ -138,6 +137,7 @@ static bool RunSexyALTest(SexyAL *interface, SexyAL_buffering *buffering, const
      buffering->buffer_size = 0;
      buffering->period_size = 0;
      buffering->latency = 0;
+     buffering->bt_gran = 0;
 
      if(!(Output=Interface->Open(Interface, device, &format, buffering, driver_type)))
      {
@@ -199,15 +199,17 @@ static bool RunSexyALTest(SexyAL *interface, SexyAL_buffering *buffering, const
 }
 #endif
 
-bool InitSound(MDFNGI *gi)
+bool Sound_Init(MDFNGI *gi)
 {
+ SexyAL_DriverInfo CurDriver;
+
+ NeedReInit = false;
  SoundRate = 0;
 
- memset(&format,0,sizeof(format));
- memset(&buffering,0,sizeof(buffering));
+ memset(&format, 0, sizeof(format));
+ memset(&buffering, 0, sizeof(buffering));
 
- Interface = (SexyAL *)SexyAL_Init(0);
- DriverTypes = Interface->EnumerateTypes(Interface);
+ Interface = new SexyAL();
 
  format.sampformat = SEXYAL_FMT_PCMS16;
 
@@ -215,95 +217,98 @@ bool InitSound(MDFNGI *gi)
  format.channels = gi->soundchan;
 
  format.revbyteorder = 0;
-
+ format.noninterleaved = false;
  format.rate = gi->soundrate ? gi->soundrate : MDFN_GetSettingUI("sound.rate");
 
  buffering.ms = MDFN_GetSettingUI("sound.buffer_time");
+
+ if(!buffering.ms)
+ {
+  buffering.overhead_kludge = true;
+  buffering.ms = 7 + floor(0.5 + 1.5 * 1000.0 / gi->fps * (256 * 65536));
+ }
+ else
+  buffering.overhead_kludge = false;
+
  buffering.period_us = MDFN_GetSettingUI("sound.period_time");
 
  std::string zedevice = MDFN_GetSettingS("sound.device");
  std::string zedriver = MDFN_GetSettingS("sound.driver");
 
- CurDriverIndex = -1;
-
- if(!strcasecmp(zedriver.c_str(), "default"))
-  CurDriverIndex = 0;
- else
- {
-  for(int x = 0; DriverTypes[x].short_name; x++)
-  {
-   if(!strcasecmp(zedriver.c_str(), DriverTypes[x].short_name))
-   {
-    CurDriverIndex = x;
-    break;
-   }
-  }
- }
-
  MDFNI_printf(_("\nInitializing sound...\n"));
  MDFN_indent(1);
 
- if(CurDriverIndex == -1)
+ if(!Interface->FindDriver(&CurDriver, zedriver.c_str()))
  {
-  MDFN_printf(_("\nUnknown sound driver \"%s\".  Supported sound drivers:\n"), zedriver.c_str());
+  std::vector<SexyAL_DriverInfo> DriverTypes = Interface->GetDriverList();
+
+  MDFN_printf(_("\nUnknown sound driver \"%s\".  Compiled-in sound drivers:\n"), zedriver.c_str());
 
   MDFN_indent(2);
-  for(int x = 0; DriverTypes[x].short_name; x++)
+  for(unsigned x = 0; x < DriverTypes.size(); x++)
   {
    MDFN_printf("%s\n", DriverTypes[x].short_name);
   }
   MDFN_indent(-2);
   MDFN_printf("\n");
-  Interface->Destroy(Interface);
+
+  delete Interface;
   Interface = NULL;
+
   MDFN_indent(-1);
   return(FALSE);
  }
 
  if(!strcasecmp(zedevice.c_str(), "default"))
-  MDFNI_printf(_("Using \"%s\" audio driver with SexyAL's default device selection."), DriverTypes[CurDriverIndex].name);
+  MDFNI_printf(_("Using \"%s\" audio driver with SexyAL's default device selection."), CurDriver.name);
  else
-  MDFNI_printf(_("Using \"%s\" audio driver with device \"%s\":"), DriverTypes[CurDriverIndex].name, zedevice.c_str());
+  MDFNI_printf(_("Using \"%s\" audio driver with device \"%s\":"), CurDriver.name, zedevice.c_str());
  MDFN_indent(1);
 
- //RunSexyALTest(Interface, &buffering, zedevice.c_str(), DriverTypes[CurDriverIndex].type);
+ //RunSexyALTest(Interface, &buffering, zedevice.c_str(), CurDriver.type);
  //exit(1);
- if(!(Output=Interface->Open(Interface, zedevice.c_str(), &format, &buffering, DriverTypes[CurDriverIndex].type)))
+ if(!(Output=Interface->Open(zedevice.c_str(), &format, &buffering, CurDriver.type)))
  {
   MDFND_PrintError(_("Error opening a sound device."));
-  Interface->Destroy(Interface);
-  Interface=0;
+
+  delete Interface;
+  Interface = NULL;
   MDFN_indent(-2);
   return(FALSE);
  }
 
- if(format.rate < 22050 || format.rate > 1048576)
+ if(format.rate < 22050 || format.rate > 192000)
  {
-  MDFND_PrintError(_("Set rate is out of range [22050-1048576]"));
-  KillSound();
+  MDFND_PrintError(_("Set rate is out of range [22050-192000]"));
+  Sound_Kill();
   MDFN_indent(-2);
   return(FALSE);
  }
- MDFNI_printf(_("\nBits: %u\nRate: %u\nChannels: %u\nByte order: CPU %s\nBuffer size: %u sample frames(%f ms)\n"), (format.sampformat>>4)*8,format.rate,format.channels,format.revbyteorder?"Reversed":"Native", buffering.buffer_size, (double)buffering.buffer_size * 1000 / format.rate);
+ MDFNI_printf(_("\nBits: %u%s\nRate: %u\nChannels: %u%s\nByte order: CPU %s\nBuffer size: %u sample frames(%f ms)\n"), (format.sampformat>>4)*8, (format.sampformat == SEXYAL_FMT_PCMFLOAT) ? _(" (floating-point)") : "",format.rate,format.channels,format.noninterleaved ? _(" (non-interleaved) ") : "", format.revbyteorder?"Reversed":"Native", buffering.buffer_size, (double)buffering.buffer_size * 1000 / format.rate);
  MDFNI_printf(_("Latency: %u sample frames(%f ms)\n"), buffering.latency, (double)buffering.latency * 1000 / format.rate);
 
  if(buffering.period_size)
  {
-  int64_t pt_test_result = ((int64_t)buffering.period_size * (1000 * 1000) / format.rate);
-
+  //int64_t pt_test_result = ((int64_t)buffering.period_size * (1000 * 1000) / format.rate);
+  int64_t bt_test_result = ((int64_t)(buffering.bt_gran ? buffering.bt_gran : buffering.period_size) * (1000 * 1000) / format.rate);
   MDFNI_printf(_("Period size: %u sample frames(%f ms)\n"), buffering.period_size, (double)buffering.period_size * 1000 / format.rate);
 
-  if(pt_test_result > 5333)
+  if(bt_test_result > 5333)
   {
    MDFN_indent(1);
-   MDFN_printf(_("Warning: Period time is too large(it should be <= ~5.333ms).  Video will appear very jerky.\n"));
+
+   if(!buffering.bt_gran)
+    MDFN_printf(_("Warning: Period time is too large(it should be <= ~5.333ms).  Video will appear very jerky.\n"));
+   else
+    MDFN_printf(_("Warning: Buffer update timing granularity is too large(%f; it should be <= ~5.333ms).  Video will appear very jerky.\n"), (double)buffering.bt_gran * 1000 / format.rate);
    MDFN_indent(-1);
   }
  }
- format.sampformat=SEXYAL_FMT_PCMS16;
- format.channels=gi->soundchan?gi->soundchan:1;
- format.revbyteorder=0;
 
+ format.sampformat = SEXYAL_FMT_PCMS16;
+ format.channels = gi->soundchan?gi->soundchan:1;
+ format.revbyteorder = 0;
+ format.noninterleaved = false;
  //format.rate=gi->soundrate?gi->soundrate:soundrate;
 
  Output->SetConvert(Output, &format);
@@ -317,12 +322,7 @@ bool InitSound(MDFNGI *gi)
  return(1);
 }
 
-void SilenceSound(int n)
-{
-
-}
-
-bool KillSound(void)
+bool Sound_Kill(void)
 {
  SoundRate = 0;
 
@@ -338,9 +338,10 @@ bool KillSound(void)
   Output->Close(Output);
 
  if(Interface)
-  Interface->Destroy(Interface);
-
- Interface = NULL;
+ {
+  delete Interface;
+  Interface = NULL;
+ }
 
  if(!Output)
   return(FALSE);
@@ -351,7 +352,7 @@ bool KillSound(void)
 }
 
 
-int16 *GetEmuModSoundBuffer(int32 *max_size_bytes)
+int16 *Sound_GetEmuModBuffer(int32 *max_size_bytes)
 {
  *max_size_bytes = EmuModBufferSize;
 
