@@ -1,8 +1,10 @@
-#ifndef MDFN_FILE_H
-#define MDFN_FILE_H
+#ifndef __MDFN_FILE_H
+#define __MDFN_FILE_H
 
 #include <string>
 
+class Stream;
+
 #define MDFNFILE_EC_NOTFOUND	1
 #define MDFNFILE_EC_OTHER	2
 
@@ -10,33 +12,16 @@ class MDFNFILE
 {
 	public:
 
-	MDFNFILE();
-	// WIP constructors:
 	MDFNFILE(const char *path, const FileExtensionSpecStruct *known_ext, const char *purpose = NULL);
-
 	~MDFNFILE();
 
-	bool Open(const char *path, const FileExtensionSpecStruct *known_ext, const char *purpose = NULL, const bool suppress_notfound_pe = FALSE);
-	INLINE bool Open(const std::string &path, const FileExtensionSpecStruct *known_ext, const char *purpose = NULL, const bool suppress_notfound_pe = FALSE)
-	{
-	 return(Open(path.c_str(), known_ext, purpose, suppress_notfound_pe));
-	}
-
-        bool ApplyIPS(FILE *);
-	bool Close(void);
+        void ApplyIPS(Stream *);
+	void Close(void) throw();
 
 	const int64 &size;
 	const uint8 * const &data;
-	const char * const &ext;
-
-	// Currently, only valid with Open()
-	inline int GetErrorCode(int *get_errno = NULL)
-	{
-	 if(get_errno)
-	  *get_errno = local_errno;
-
-	 return(error_code);
-	}
+        const char * const &ext;
+        const char * const &fbase;
 
 	inline int64 Size(void)
 	{
@@ -84,9 +69,7 @@ class MDFNFILE
         uint8 *f_data;
         int64 f_size;
         char *f_ext;
-
-	int error_code;
-	int local_errno;
+	char *f_fbase;
 
         int64 location;
 
@@ -94,25 +77,10 @@ class MDFNFILE
 	bool is_mmap;
 	#endif
 
-	bool MakeMemWrapAndClose(void *tz, int type);
+	void Open(const char *path, const FileExtensionSpecStruct *known_ext, const char *purpose = NULL);
+	void MakeMemWrap(void *tz, int type);
 };
 
-#if 0
-MDFNFILE *MDFN_fopen(const char *path, const char *ipsfn, const char *mode, const FileExtensionSpecStruct *known_ext);
-int MDFN_fclose(MDFNFILE*);
-uint64 MDFN_fread(void *ptr, size_t size, size_t nmemb, MDFNFILE*);
-uint64 MDFN_fwrite(void *ptr, size_t size, size_t nmemb, MDFNFILE*);
-int MDFN_fseek(MDFNFILE*, int64 offset, int whence);
-uint64 MDFN_ftell(MDFNFILE*);
-void MDFN_rewind(MDFNFILE*);
-int MDFN_read32le(uint32 *Bufo, MDFNFILE*);
-int MDFN_read16le(uint16 *Bufo, MDFNFILE*);
-int MDFN_fgetc(MDFNFILE*);
-uint64 MDFN_fgetsize(MDFNFILE*);
-int MDFN_fisarchive(MDFNFILE*);
-char *MDFN_fgets(char *s, int size, MDFNFILE *);
-#endif
-
 class PtrLengthPair
 {
  public:
