@@ -54,8 +54,6 @@ found freely through public domain sources.
 #include "v810_cpu.h"
 #include "v810_cpuD.h"
 
-//#include "fpu-new/softfloat.h"
-
 V810::V810()
 {
  #ifdef WANT_DEBUGGER
@@ -90,7 +88,7 @@ V810::V810()
 
 V810::~V810()
 {
-
+ Kill();
 }
 
 INLINE void V810::RecalcIPendingCache(void)
@@ -229,7 +227,13 @@ INLINE uint32 V810::RDCACHE(v810_timestamp_t &timestamp, uint32 addr)
    else
    {
     timestamp++;
-    Cache[CI].data[SBI] = MemRead16(timestamp, addr & ~0x3) | ((MemRead16(timestamp, (addr & ~0x3) | 0x2) << 16));
+
+    uint32 tmp;
+
+    tmp = MemRead16(timestamp, addr & ~0x3);
+    tmp |= MemRead16(timestamp, (addr & ~0x3) | 0x2) << 16;
+
+    Cache[CI].data[SBI] = tmp;
    }
    Cache[CI].data_valid[SBI] = TRUE;
   }
@@ -244,7 +248,13 @@ INLINE uint32 V810::RDCACHE(v810_timestamp_t &timestamp, uint32 addr)
   else
   {
    timestamp++;
-   Cache[CI].data[SBI] = MemRead16(timestamp, addr & ~0x3) | ((MemRead16(timestamp, (addr & ~0x3) | 0x2) << 16));
+
+   uint32 tmp;
+
+   tmp = MemRead16(timestamp, addr & ~0x3);
+   tmp |= MemRead16(timestamp, (addr & ~0x3) | 0x2) << 16;
+
+   Cache[CI].data[SBI] = tmp;
   }
   //Cache[CI].data[SBI] = MemRead32(timestamp, addr & ~0x3);
   Cache[CI].data_valid[SBI] = TRUE;
@@ -368,10 +378,7 @@ uint8 *V810::SetFastMap(uint32 addresses[], uint32 length, unsigned int num_addr
  }
  assert((length & (V810_FAST_MAP_PSIZE - 1)) == 0);
 
- if(!(ret = (uint8 *)MDFN_malloc(length + V810_FAST_MAP_TRAMPOLINE_SIZE, name)))
- {
-  return(NULL);
- }
+ ret = (uint8 *)MDFN_malloc_T(length + V810_FAST_MAP_TRAMPOLINE_SIZE, name);
 
  for(unsigned int i = length; i < length + V810_FAST_MAP_TRAMPOLINE_SIZE; i += 2)
  {
@@ -1114,46 +1121,26 @@ INLINE void V810::SetFPUOPNonFPUFlags(uint32 result)
                  //printf("MEOW: %08x\n", S_REG[PSW] & (PSW_S | PSW_CY));
 }
 
-INLINE bool V810::CheckFPInputException(uint32 fpval)
-{
- // Zero isn't a subnormal! (OR IS IT *DUN DUN DUNNN* ;b)
- if(!(fpval & 0x7FFFFFFF))
-  return(false);
-
- switch((fpval >> 23) & 0xFF)
- {
-  case 0x00: // Subnormal		
-  case 0xFF: // NaN or infinity
-	{
-	 //puts("New FPU FRO");
-
-	 S_REG[PSW] |= PSW_FRO;
-
-	 SetPC(GetPC() - 4);
-	 Exception(FPU_HANDLER_ADDR, ECODE_FRO);
-	}
-	return(true);	// Yes, exception occurred
- }
- return(false);	// No, no exception occurred.
-}
-
 bool V810::FPU_DoesExceptionKillResult(void)
 {
- if(float_exception_flags & float_flag_invalid)
+ const uint32 float_exception_flags = fpo.get_flags();
+
+ if(float_exception_flags & V810_FP_Ops::flag_reserved)
   return(true);
 
- if(float_exception_flags & float_flag_divbyzero)
+ if(float_exception_flags & V810_FP_Ops::flag_invalid)
+  return(true);
+
+ if(float_exception_flags & V810_FP_Ops::flag_divbyzero)
   return(true);
 
 
  // Return false here, so that the result of this calculation IS put in the output register.
- // (Incidentally, to get the result of operations on overflow to match a real V810, required a bit of hacking of the SoftFloat code to "wrap" the exponent
- // on overflow,
- // rather than generating an infinity.  The wrapping behavior is specified in IEE 754 AFAIK, and is useful in cases where you divide a huge number
+ // Wrap the exponent on overflow, rather than generating an infinity.  The wrapping behavior is specified in IEE 754 AFAIK,
+ // and is useful in cases where you divide a huge number
  // by another huge number, and fix the result afterwards based on the number of overflows that occurred.  Probably requires some custom assembly code,
  // though.  And it's the kind of thing you'd see in an engineering or physics program, not in a perverted video game :b).
- // Oh, and just a note to self, FPR is NOT set when an overflow occurs.  Or it is in certain cases?
- if(float_exception_flags & float_flag_overflow)
+ if(float_exception_flags & V810_FP_Ops::flag_overflow)
   return(false);
 
  return(false);
@@ -1161,10 +1148,20 @@ bool V810::FPU_DoesExceptionKillResult(void)
 
 void V810::FPU_DoException(void)
 {
- if(float_exception_flags & float_flag_invalid)
- {
-  //puts("New FPU Invalid");
+ const uint32 float_exception_flags = fpo.get_flags();
 
+ if(float_exception_flags & V810_FP_Ops::flag_reserved)
+ {
+  S_REG[PSW] |= PSW_FRO;
+
+  SetPC(GetPC() - 4);
+  Exception(FPU_HANDLER_ADDR, ECODE_FRO);
+
+  return;
+ }
+
+ if(float_exception_flags & V810_FP_Ops::flag_invalid)
+ {
   S_REG[PSW] |= PSW_FIV;
 
   SetPC(GetPC() - 4);
@@ -1173,10 +1170,8 @@ void V810::FPU_DoException(void)
   return;
  }
 
- if(float_exception_flags & float_flag_divbyzero)
+ if(float_exception_flags & V810_FP_Ops::flag_divbyzero)
  {
-  //puts("New FPU Divide by Zero");
-
   S_REG[PSW] |= PSW_FZD;
 
   SetPC(GetPC() - 4);
@@ -1185,24 +1180,21 @@ void V810::FPU_DoException(void)
   return;
  }
 
- if(float_exception_flags & float_flag_underflow)
+ if(float_exception_flags & V810_FP_Ops::flag_underflow)
  {
-  //puts("New FPU Underflow");
-
   S_REG[PSW] |= PSW_FUD;
  }
 
- if(float_exception_flags & float_flag_inexact)
+ if(float_exception_flags & V810_FP_Ops::flag_inexact)
  {
   S_REG[PSW] |= PSW_FPR;
-  //puts("New FPU Precision Degradation");
  }
 
+ //
  // FPR can be set along with overflow, so put the overflow exception handling at the end here(for Exception() messes with PSW).
- if(float_exception_flags & float_flag_overflow)
+ //
+ if(float_exception_flags & V810_FP_Ops::flag_overflow)
  {
-  //puts("New FPU Overflow");
-
   S_REG[PSW] |= PSW_FOV;
 
   SetPC(GetPC() - 4);
@@ -1218,38 +1210,19 @@ bool V810::IsSubnormal(uint32 fpval)
  return(false);
 }
 
-INLINE void V810::FPU_Math_Template(float32 (*func)(float32, float32), uint32 arg1, uint32 arg2)
+INLINE void V810::FPU_Math_Template(uint32 (V810_FP_Ops::*func)(uint32, uint32), uint32 arg1, uint32 arg2)
 {
- if(CheckFPInputException(P_REG[arg1]) || CheckFPInputException(P_REG[arg2]))
+ uint32 result;
+
+ fpo.clear_flags();
+ result = (fpo.*func)(P_REG[arg1], P_REG[arg2]);
+
+ if(!FPU_DoesExceptionKillResult())
  {
-
- }
- else
- {
-  uint32 result;
-
-  float_exception_flags = 0;
-  result = func(P_REG[arg1], P_REG[arg2]);
-
-  if(IsSubnormal(result))
-  {
-   float_exception_flags |= float_flag_underflow;
-   float_exception_flags |= float_flag_inexact;
-  }
-
-  //printf("Result: %08x, %02x; %02x\n", result, (result >> 23) & 0xFF, float_exception_flags);
-
-  if(!FPU_DoesExceptionKillResult())
-  {
-   // Force it to +/- zero before setting S/Z based off of it(confirmed with subf.s on real V810, at least).
-   if(float_exception_flags & float_flag_underflow)
-    result &= 0x80000000;
-
-   SetFPUOPNonFPUFlags(result);
-   SetPREG(arg1, result);
-  }
-  FPU_DoException();
+  SetFPUOPNonFPUFlags(result);
+  SetPREG(arg1, result);
  }
+ FPU_DoException();
 }
 
 void V810::fpu_subop(v810_timestamp_t &timestamp, int sub_op, int arg1, int arg2)
@@ -1310,34 +1283,25 @@ void V810::fpu_subop(v810_timestamp_t &timestamp, int sub_op, int arg1, int arg2
 		{
 		 uint32 result;
 
-                 float_exception_flags = 0;
-		 result = int32_to_float32((int32)P_REG[arg2]);
+                 fpo.clear_flags();
+		 result = fpo.itof(P_REG[arg2]);
 
 		 if(!FPU_DoesExceptionKillResult())
 		 {
 		  SetPREG(arg1, result);
 		  SetFPUOPNonFPUFlags(result);
 		 }
-		 else
-		 {
-		  puts("Exception on CVT.WS?????");	// This shouldn't happen, but just in case there's a bug...
-		 }
 		 FPU_DoException();
 		}
 		break;	// End CVT.WS
 
 	case CVT_SW:
 		timestamp += 8;
-                if(CheckFPInputException(P_REG[arg2]))
-                {
-
-                }
-		else
 		{
 		 int32 result;
 
-                 float_exception_flags = 0;
-		 result = float32_to_int32(P_REG[arg2]);
+                 fpo.clear_flags();
+		 result = fpo.ftoi(P_REG[arg2], false);
 
 		 if(!FPU_DoesExceptionKillResult())
 		 {
@@ -1350,69 +1314,45 @@ void V810::fpu_subop(v810_timestamp_t &timestamp, int sub_op, int arg1, int arg2
 		break;	// End CVT.SW
 
 	case ADDF_S: timestamp += 8;
-		     FPU_Math_Template(float32_add, arg1, arg2);
+		     FPU_Math_Template(&V810_FP_Ops::add, arg1, arg2);
 		     break;
 
 	case SUBF_S: timestamp += 11;
-		     FPU_Math_Template(float32_sub, arg1, arg2);
+		     FPU_Math_Template(&V810_FP_Ops::sub, arg1, arg2);
 		     break;
 
         case CMPF_S: timestamp += 6;
 		     // Don't handle this like subf.s because the flags
 		     // have slightly different semantics(mostly regarding underflow/subnormal results) (confirmed on real V810).
-                     if(CheckFPInputException(P_REG[arg1]) || CheckFPInputException(P_REG[arg2]))
-                     {
-
-                     }
-		     else
+		     fpo.clear_flags();
 		     {
-		      SetFlag(PSW_OV, 0);
+		      int32 result;
 
-		      if(float32_eq(P_REG[arg1], P_REG[arg2]))
-		      {
-		       SetFlag(PSW_Z, 1);
-		       SetFlag(PSW_S, 0);
-		       SetFlag(PSW_CY, 0);
-		      }
-		      else
-		      {
-		       SetFlag(PSW_Z, 0);
+		      result = fpo.cmp(P_REG[arg1], P_REG[arg2]);
 
-		       if(float32_lt(P_REG[arg1], P_REG[arg2]))
-		       {
-		        SetFlag(PSW_S, 1);
-		        SetFlag(PSW_CY, 1);
-		       }
-		       else
-		       {
-		        SetFlag(PSW_S, 0);
-		        SetFlag(PSW_CY, 0);
-                       }
+	              if(!FPU_DoesExceptionKillResult())
+		      {
+		       SetFPUOPNonFPUFlags(result);
 		      }
-		     }	// else of if(CheckFP...
+		      FPU_DoException();
+		     }
                      break;
 
 	case MULF_S: timestamp += 7;
-		     FPU_Math_Template(float32_mul, arg1, arg2);
+		     FPU_Math_Template(&V810_FP_Ops::mul, arg1, arg2);
 		     break;
 
 	case DIVF_S: timestamp += 43;
-		     FPU_Math_Template(float32_div, arg1, arg2);
+		     FPU_Math_Template(&V810_FP_Ops::div, arg1, arg2);
 		     break;
 
 	case TRNC_SW:
                 timestamp += 7;
-
-		if(CheckFPInputException(P_REG[arg2]))
-		{
-
-		}
-		else
                 {
                  int32 result;
 
-		 float_exception_flags = 0;
-                 result = float32_to_int32_round_to_zero(P_REG[arg2]);
+		 fpo.clear_flags();
+                 result = fpo.ftoi(P_REG[arg2], true);
 
                  if(!FPU_DoesExceptionKillResult())
                  {
