@@ -26,7 +26,8 @@
 #ifndef FUSE_Z80_H
 #define FUSE_Z80_H
 
-#include "../../mednafen.h"
+#include <mednafen/mednafen.h>
+#include <mednafen/state.h>
 
 /* Union allowing a register pair to be accessed as bytes or as a word */
 typedef union {
@@ -116,8 +117,6 @@ static INLINE uint8 Z80_RP_MACRO(uint16 A)
 
 
 void z80_set_interrupt(int set);
-
-#include "../../mednafen/state.h"
 int z80_state_action(StateMem *sm, int load, int data_only, const char *section_name);
 
 #endif			/* #ifndef FUSE_Z80_H */
