@@ -172,7 +172,7 @@ enum
 	MIKIE_PIXEL_FORMAT_32BPP,
 };
 
-#include <blip/Stereo_Buffer.h>
+#include <mednafen/sound/Stereo_Buffer.h>
 
 typedef Blip_Synth<blip_good_quality, 256 * 4> Synth;
 
