@@ -1,7 +1,23 @@
 #ifndef __MDFN_MATH_OPS_H
 #define __MDFN_MATH_OPS_H
 
-uint32 round_up_pow2(uint32 v);
+// Source: http://graphics.stanford.edu/~seander/bithacks.html#RoundUpPowerOf2
+// Rounds up to the nearest power of 2.
+static INLINE uint64 round_up_pow2(uint64 v)
+{
+ v--;
+ v |= v >> 1;
+ v |= v >> 2;
+ v |= v >> 4;
+ v |= v >> 8;
+ v |= v >> 16;
+ v |= v >> 32;
+ v++;
+
+ v += (v == 0);
+
+ return(v);
+}
 
 static INLINE uint32 uilog2(uint32 v)
 {
@@ -38,7 +54,34 @@ static INLINE uint32 uilog2(uint32 v)
 // convert those faster with typecasts...
 #define sign_x_to_s32(_bits, _value) (((int32)((uint32)(_value) << (32 - _bits))) >> (32 - _bits))
 
+static INLINE int32 clamp_to_u8(int32 i)
+{
+ if(i & 0xFFFFFF00)
+  i = (((~i) >> 30) & 0xFF);
 
-#define INT16_TO_BCD(A)  ((((((A) % 100) / 10) * 16 + ((A) % 10))) | (((((((A) / 100) % 100) / 10) * 16 + (((A) / 100) % 10))) << 8))   // convert INT16 --> BCD
+ return(i);
+}
+
+static INLINE int32 clamp_to_u16(int32 i)
+{
+ if(i & 0xFFFF0000)
+  i = (((~i) >> 31) & 0xFFFF);
+
+ return(i);
+}
+
+template<typename T, typename U, typename V> static INLINE void clamp(T *val, U minimum, V maximum)
+{
+ if(*val < minimum)
+ {
+  //printf("Warning: clamping to minimum(%d)\n", (int)minimum);
+  *val = minimum;
+ }
+ if(*val > maximum)
+ {
+  //printf("Warning: clamping to maximum(%d)\n", (int)maximum);
+  *val = maximum;
+ }
+}
 
 #endif
