@@ -220,20 +220,20 @@ void MDVDP::vdp_ctrl_w(uint16 data)
                 case 0x00: /* V bus to VDP DMA */
                 case 0x40: /* V bus to VDP DMA */
 		    if((code & 0x0F) != 0x1 && (code & 0x0F) != 0x3 && (code & 0x0F) != 0x5)
-                     printf("Invalid code for V bus to VDP dma: %02x\n", code);
+                     MD_DBG(MD_DBG_WARNING, "[VDP] Invalid code for V bus to VDP dma: %02x\n", code);
 		    status |= 0x2;
 		    Recalc68KSuspend();
                     break;
 
                 case 0x80: /* VRAM fill */
                     if((code & 0x0F) != 0x1)
-                     printf("Invalid code for fill dma: %02x\n", code);
+                     MD_DBG(MD_DBG_WARNING, "[VDP] Invalid code for fill dma: %02x\n", code);
                     dma_fill = 1;
                     break;
 
                 case 0xC0: /* VRAM copy */
                     if((code & 0x0F) != 0x0)
-                     printf("Invalid code for copy dma: %02x\n", code);
+                     MD_DBG(MD_DBG_WARNING, "[VDP] Invalid code for copy dma: %02x\n", code);
 		    status |= 0x2;
                     break;
             }
@@ -965,7 +965,7 @@ void MDVDP::render_line(int line)
         if(reg[12] & 8)
         {
             merge(&nta_buf[0x20], &ntb_buf[0x20], &bg_buf[0x20], lut[2], (reg[12] & 1) ? 320 : 256);
-            memset(&obj_buf[0x20], 0, (reg[12] & 1) ? 320 : 256);
+            memset(&obj_buf[0], 0, 0x20 + ((reg[12] & 1) ? 320 : 256) + 0x20);	// Need to clear left and right padding areas to prevent uninitialized memory usage.
 
             if(im2_flag)
                 render_obj_im2(line, obj_buf, lut[3]);
@@ -979,8 +979,15 @@ void MDVDP::render_line(int line)
         }
         else
         {
+	  // So sprite rendering won't read from uninitialized memory.
+	  memset(&lb[0x00], 0, 0x20);
+	  memset(&lb[0x20 + ((reg[12] & 1) ? 320 : 256)], 0, 0x20);
+
+
 	  if(UserLE & 0x4)
            merge(&nta_buf[0x20], &ntb_buf[0x20], &lb[0x20], lut[0], (reg[12] & 1) ? 320 : 256);
+	  else
+	   memset(&lb[0x20], 0, (reg[12] & 1) ? 320 : 256);
 
           if(im2_flag)
            render_obj_im2(line, lb, lut[1]);
@@ -1773,7 +1780,6 @@ void MDVDP::render_obj(int line, uint8 *buf, uint8 *table)
     int pixellimit = (reg[12] & 1) ? 320 : 256;
     int pixelcount = 0;
     int width;
-    int height;
     int v_line;
     int column;
     int sol_flag = 0;
@@ -1811,7 +1817,6 @@ void MDVDP::render_obj(int line, uint8 *buf, uint8 *table)
             attr = object_info[count].attr;
             attr_mask = (attr & 0x1800);
 
-            height = sizetab[size & 3];
             palette = (attr >> 9) & 0x70;
 
             v_line = (line - ypos);
@@ -1822,8 +1827,6 @@ void MDVDP::render_obj(int line, uint8 *buf, uint8 *table)
             s = &name_lut[((attr >> 3) & 0x300) | (size << 4) | (nt_row << 2)];
 
             lb = (uint8 *)&buf[0x20 + (xpos - 0x80)];
-	    if((0x20 + (xpos - 0x80)) < 0 || xpos >= 0x300)
-		printf("BLAH: %d\n", 0x20 + (xpos - 0x80));
             width >>= 3;
             for(column = 0; column < width; column += 1, lb+=8)
             {
@@ -1848,7 +1851,6 @@ void MDVDP::render_obj_im2(int line, uint8 *buf, uint8 *table)
     int pixellimit = (reg[12] & 1) ? 320 : 256;
     int pixelcount = 0;
     int width;
-    int height;
     int v_line;
     int column;
     int sol_flag = 0;
@@ -1887,7 +1889,6 @@ void MDVDP::render_obj_im2(int line, uint8 *buf, uint8 *table)
             attr = object_info[count].attr;
             attr_mask = (attr & 0x1800);
 
-            height = sizetab[size & 3];
             palette = (attr >> 9) & 0x70;
 
             v_line = (line - ypos);
