@@ -1,4 +1,5 @@
-#ifndef _MEDNAFEN_H
+#ifndef __MDFN_MEDNAFEN_H
+#define __MDFN_MEDNAFEN_H
 
 #include "types.h"
 #include <stdio.h>
@@ -15,11 +16,11 @@ extern MDFNGI *MDFNGameInfo;
 
 #include "settings.h"
 
-void MDFN_PrintError(const char *format, ...) throw() MDFN_FORMATSTR(printf, 1, 2);
-void MDFN_printf(const char *format, ...) throw() MDFN_FORMATSTR(printf, 1, 2);
-void MDFN_DispMessage(const char *format, ...) throw() MDFN_FORMATSTR(printf, 1, 2);
+void MDFN_PrintError(const char *format, ...) throw() MDFN_FORMATSTR(gnu_printf, 1, 2);
+void MDFN_printf(const char *format, ...) throw() MDFN_FORMATSTR(gnu_printf, 1, 2);
+void MDFN_DispMessage(const char *format, ...) throw() MDFN_FORMATSTR(gnu_printf, 1, 2);
 
-void MDFN_DebugPrintReal(const char *file, const int line, const char *format, ...) MDFN_FORMATSTR(printf, 3, 4);
+void MDFN_DebugPrintReal(const char *file, const int line, const char *format, ...) MDFN_FORMATSTR(gnu_printf, 3, 4);
 
 #define MDFN_DebugPrint(format, ...) MDFN_DebugPrintReal(__FILE__, __LINE__, format, ## __VA_ARGS__)
 
@@ -38,5 +39,4 @@ int MDFN_RawInputStateAction(StateMem *sm, int load, int data_only);
 #include "endian.h"
 #include "memory.h"
 
-#define _MEDNAFEN_H
 #endif
