@@ -80,13 +80,13 @@
 		timekeeping variables will get out of sync and everything will go boom.
 */
 
-#include "../mednafen.h"
-#include "../cdrom/cdromif.h"
-#include "../cdrom/scsicd.h"
-#include "../okiadpcm.h"
+#include <mednafen/mednafen.h>
+#include <mednafen/cdrom/cdromif.h>
+#include <mednafen/cdrom/scsicd.h>
+#include <mednafen/sound/okiadpcm.h>
+#include <mednafen/cdrom/SimpleFIFO.h>
 
 #include "pcecd.h"
-#include "../cdrom/SimpleFIFO.h"
 
 //#define PCECD_DEBUG
 
@@ -219,9 +219,6 @@ static int32 CalcNextEvent(int32 base)
  return(next_event);
 }
 
-
-#ifndef __MDFN_INCLUDED_FROM_PCE_FAST
-
 uint32 PCECD_GetRegister(const unsigned int id, char *special, const uint32 special_len)
 {
  uint32 value = 0xDEADBEEF;
@@ -324,8 +321,6 @@ void ADPCM_PokeRAM(uint32 Address, uint32 Length, const uint8 *Buffer)
  }
 }
 
-#endif
-
 static void update_irq_state()
 {
         uint8           irq = _Port[2] & _Port[0x3] & (0x4|0x8|0x10|0x20|0x40);
@@ -419,8 +414,20 @@ bool PCECD_SetSettings(const PCECD_Settings *settings)
 	return true;
 }
 
-bool PCECD_Init(const PCECD_Settings *settings, void (*irqcb)(bool), double master_clock, int32* adbuf, int32* hrbuf_l, int32* hrbuf_r)
+static void Cleanup(void)
 {
+        if(ADPCM.RAM)
+        {
+         MDFN_free(ADPCM.RAM);
+         ADPCM.RAM = NULL;
+	}
+ 	SCSICD_Close();
+}
+
+void PCECD_Init(const PCECD_Settings *settings, void (*irqcb)(bool), double master_clock, int32* adbuf, int32* hrbuf_l, int32* hrbuf_r)
+{
+ try
+ {
 	ADPCM.last_pcm = 0;
 	ADPCM.integrate_accum = 0;
 	ADPCM.lp1p_fstate = 0;
@@ -434,27 +441,23 @@ bool PCECD_Init(const PCECD_Settings *settings, void (*irqcb)(bool), double mast
 
 	SCSICD_Init(SCSICD_PCE, 3, hrbuf_l, hrbuf_r, 126000, master_clock, CDIRQ, StuffSubchannel);
 
-        if(!(ADPCM.RAM = (uint8 *)MDFN_malloc(0x10000, _("PCE ADPCM RAM"))))
-        {
-         return(0);
-        }
+        ADPCM.RAM = (uint8 *)MDFN_malloc_T(0x10000, _("PCE ADPCM RAM"));
 
 	PCECD_SetSettings(settings);
 
         ADPCM.bigdivacc = (int64)((double)master_clock * 65536 / 32087.5);
-
-	return(TRUE);
+ }
+ catch(...)
+ {
+  Cleanup();
+  throw;
+ }
 }
 
 
 void PCECD_Close(void)
 {
-        if(ADPCM.RAM)
-        {
-         MDFN_free(ADPCM.RAM);
-         ADPCM.RAM = NULL;
-        }
-	SCSICD_Close();
+	Cleanup();
 }
 
 
@@ -1196,11 +1199,7 @@ static INLINE void ADPCM_Run(const int32 clocks, const int32 timestamp)
 
 // The return value of this function is ignored in PCECD_Read() and PCECD_Write() for speed reasons, and the
 // fact that reading and writing can change the next potential event,
-#ifdef __MDFN_INCLUDED_FROM_PCE_FAST
-void PCECD_Run(uint32 in_timestamp)
-#else
 int32 PCECD_Run(uint32 in_timestamp)
-#endif
 {
  int32 clocks = in_timestamp - lastts;
  int32 running_ts = lastts;
@@ -1245,10 +1244,7 @@ int32 PCECD_Run(uint32 in_timestamp)
  //puts("Run End");
  //fflush(stdout);
 
-
- #ifndef __MDFN_INCLUDED_FROM_PCE_FAST
  return(CalcNextEvent(0x7FFFFFFF));
- #endif
 }
 
 void PCECD_ResetTS(uint32 ts_base)
