@@ -18,8 +18,8 @@
 #include "pce.h"
 #include "input.h"
 #include "huc.h"
-#include "../movie.h"
-#include "../endian.h"
+#include <mednafen/movie.h>
+#include <mednafen/endian.h>
 
 namespace PCE_Fast
 {
