@@ -45,7 +45,7 @@ void FXINPUT_Frame(void);
 int FXINPUT_StateAction(StateMem *sm, int load, int data_only);
 
 v810_timestamp_t FXINPUT_Update(const v810_timestamp_t timestamp);
-void FXINPUT_ResetTS(void);
+void FXINPUT_ResetTS(int32 ts_base);
 
 extern InputInfoStruct PCFXInputInfo;
 
