@@ -25,11 +25,11 @@
 #include "rainbow.h"
 #include "huc6273.h"
 #include "fxscsi.h"
-#include "../cdrom/scsicd.h"
-#include "../mempatcher.h"
-#include "../cdrom/cdromif.h"
-#include "../md5.h"
-#include "../clamp.h"
+#include <mednafen/cdrom/scsicd.h>
+#include <mednafen/mempatcher.h>
+#include <mednafen/cdrom/cdromif.h>
+#include <mednafen/md5.h>
+#include <mednafen/FileStream.h>
 
 #include <trio/trio.h>
 #include <errno.h>
@@ -68,6 +68,17 @@ static uint8 ExBusReset; // I/O Register at 0x0700
 
 static bool BRAMDisabled;	// Cached at game load, don't remove this caching behavior or save game loss may result(if we ever get a GUI).
 
+//
+// Set BackupSignalDirty to true on writes to BackupRAM[] and ExBackupRAM[], and on save state load.
+// Set to false on save game load, and on periodic evaluation during emulation.
+//
+// Set BackupSaveDelay to 0 on save game load.
+//
+static bool BackupSignalDirty;
+static uint32 BackupSaveDelay;
+
+static void SaveBackupMemory(void);
+
 // Checks to see if this main-RAM-area access
 // is in the same DRAM page as the last access.
 #define RAMLPCHECK	\
@@ -225,9 +236,6 @@ static CDGameEntry GameList[] =
 
 static void Emulate(EmulateSpecStruct *espec)
 {
- v810_timestamp_t v810_timestamp;
- v810_timestamp_t new_base_ts;
-
  //printf("%d\n", PCFX_V810.v810_timestamp);
 
  FXINPUT_Frame();
@@ -243,6 +251,7 @@ static void Emulate(EmulateSpecStruct *espec)
 
  KING_StartFrame(fx_vdc_chips, espec);	//espec->surface, &espec->DisplayRect, espec->LineWidths, espec->skip);
 
+ v810_timestamp_t v810_timestamp;
  v810_timestamp = PCFX_V810.Run(pcfx_event_handler);
 
 
@@ -251,12 +260,19 @@ static void Emulate(EmulateSpecStruct *espec)
  // Call before resetting v810_timestamp
  ForceEventUpdates(v810_timestamp);
 
+ //
+ // Call KING_EndFrame() before SoundBox_Flush(), otherwise CD-DA audio distortion will occur due to sound data being updated
+ // after it was needed instead of before.
+ //
+ KING_EndFrame(v810_timestamp);
+
  //
  // new_base_ts is guaranteed to be <= v810_timestamp
  //
+ v810_timestamp_t new_base_ts;
  espec->SoundBufSize = SoundBox_Flush(v810_timestamp, &new_base_ts, espec->SoundBuf, espec->SoundBufMaxSize);
 
- KING_EndFrame(v810_timestamp, new_base_ts);
+ KING_ResetTS(new_base_ts);
  FXTIMER_ResetTS(new_base_ts);
  FXINPUT_ResetTS(new_base_ts);
  SoundBox_ResetTS(new_base_ts);
@@ -267,6 +283,35 @@ static void Emulate(EmulateSpecStruct *espec)
  espec->MasterCycles = v810_timestamp - new_base_ts;
 
  PCFX_V810.ResetTS(new_base_ts);
+
+ //
+ //
+ //
+ if(BackupSignalDirty)
+ {
+  BackupSaveDelay = 120;
+  BackupSignalDirty = false;
+ }
+ else if(BackupSaveDelay)
+ {
+  BackupSaveDelay--;
+
+  if(!BackupSaveDelay)
+  {
+   //puts("SAVE");
+   try
+   {
+    SaveBackupMemory();
+   }
+   catch(std::exception &e)
+   {
+    MDFN_PrintError(_("Error saving save-game memory: %s"), e.what());
+    MDFN_DispMessage(_("Error saving save-game memory: %s"), e.what());
+    BackupSaveDelay = 60 * 60;	// Try it again in about 60 seconds emulated time(unless more writes occur to the backup memory before then, then the regular delay
+				// will be used from that time).
+   }
+  }
+ }
 }
 
 static void PCFX_Reset(void)
@@ -422,12 +467,18 @@ static void PCFXDBG_PutAddressSpaceBytes(const char *name, uint32 Address, uint3
    else if(Address >= 0xE0000000 && Address <= 0xE7FFFFFF)
    {
     if(!(Address & 1))
+    {
+     BackupSignalDirty |= (BackupRAM[(Address & 0xFFFF) >> 1] != *Buffer);
      BackupRAM[(Address & 0xFFFF) >> 1] = *Buffer;
+    }
    }
    else if(Address >= 0xE8000000 && Address <= 0xE9FFFFFF)
    {
     if(!(Address & 1))
+    {
+     BackupSignalDirty |= (ExBackupRAM[(Address & 0xFFFF) >> 1] != *Buffer);
      ExBackupRAM[(Address & 0xFFFF) >> 1] = *Buffer;
+    }
    }
    Address++;
    Buffer++;
@@ -448,6 +499,7 @@ static void PCFXDBG_PutAddressSpaceBytes(const char *name, uint32 Address, uint3
   while(Length--)
   {
    Address &= 0x7FFF;
+   BackupSignalDirty |= (BackupRAM[Address] != *Buffer);
    BackupRAM[Address] = *Buffer;
    Address++;
    Buffer++;
@@ -458,6 +510,7 @@ static void PCFXDBG_PutAddressSpaceBytes(const char *name, uint32 Address, uint3
   while(Length--)
   {
    Address &= 0x7FFF;
+   BackupSignalDirty |= (ExBackupRAM[Address] != *Buffer);
    ExBackupRAM[Address] = *Buffer;
    Address++;
    Buffer++;
@@ -490,20 +543,35 @@ static void VDCB_IRQHook(bool asserted)
 
 static void SetRegGroups(void);
 
-static bool LoadCommon(std::vector<CDIF *> *CDInterfaces)
+static void Cleanup(void)
+{
+ for(int i = 0; i < 2; i++)
+ {
+  if(fx_vdc_chips[i])
+  {
+   delete fx_vdc_chips[i];
+   fx_vdc_chips[i] = NULL;
+  }
+ }
+
+ RAINBOW_Close();
+ KING_Close();
+ SoundBox_Kill();
+ PCFX_V810.Kill();
+
+ // The allocated memory RAM and BIOSROM is free'd in V810_Kill()
+ RAM = NULL;
+ BIOSROM = NULL;
+}
+
+static void LoadCommon(std::vector<CDIF *> *CDInterfaces)
 {
- std::string biospath = MDFN_MakeFName(MDFNMKF_FIRMWARE, 0, MDFN_GetSettingS("pcfx.bios").c_str());
- std::string fxscsi_path = MDFN_GetSettingS("pcfx.fxscsi");	// For developers only, so don't make it convenient.
- MDFNFILE BIOSFile;
  V810_Emu_Mode cpu_mode;
 
  #ifdef WANT_DEBUGGER
  SetRegGroups();
  #endif
 
- if(!BIOSFile.Open(biospath, NULL, "BIOS"))
-  return(0);
-
  cpu_mode = (V810_Emu_Mode)MDFN_GetSettingI("pcfx.cpu_emulation");
  if(cpu_mode == _V810_EMU_MODE_COUNT)
  {
@@ -521,50 +589,33 @@ static bool LoadCommon(std::vector<CDIF *> *CDInterfaces)
  uint32 RAM_Map_Addresses[1] = { 0x00000000 };
  uint32 BIOSROM_Map_Addresses[1] = { 0xFFF00000 };
 
- // todo: cleanup on error
- if(!(RAM = PCFX_V810.SetFastMap(RAM_Map_Addresses, 0x00200000, 1, _("RAM"))))
+ RAM = PCFX_V810.SetFastMap(RAM_Map_Addresses, 0x00200000, 1, _("RAM"));
+ BIOSROM = PCFX_V810.SetFastMap(BIOSROM_Map_Addresses, 0x00100000, 1, _("BIOS ROM"));
+
  {
-  return(0);
+  std::string biospath = MDFN_MakeFName(MDFNMKF_FIRMWARE, 0, MDFN_GetSettingS("pcfx.bios").c_str());
+  MDFNFILE BIOSFile(biospath.c_str(), NULL, "BIOS");
+
+  if(BIOSFile.Size() != 1024 * 1024)
+   throw MDFN_Error(0, _("BIOS ROM file is incorrect size.\n"));
+
+  memcpy(BIOSROM, BIOSFile.Data(), 1024 * 1024);
+
+  BIOSFile.Close();
  }
 
- if(!(BIOSROM = PCFX_V810.SetFastMap(BIOSROM_Map_Addresses, 0x00100000, 1, _("BIOS ROM"))))
  {
-  return(0);
- }
+  std::string fxscsi_path = MDFN_GetSettingS("pcfx.fxscsi");	// For developers only, so don't make it convenient.
 
- if(BIOSFile.Size() != 1024 * 1024)
- {
-  MDFN_PrintError(_("BIOS ROM file is incorrect size.\n"));
-  return(0);
- }
-
- memcpy(BIOSROM, BIOSFile.Data(), 1024 * 1024);
-
- BIOSFile.Close();
-
- if(fxscsi_path != "0" && fxscsi_path != "" && fxscsi_path != "none")
- {
-  MDFNFILE FXSCSIFile;
-
-  if(!FXSCSIFile.Open(fxscsi_path, NULL, "FX-SCSI ROM"))
-   return(0);
-
-  if(FXSCSIFile.Size() != 1024 * 512)
+  if(fxscsi_path != "0" && fxscsi_path != "" && fxscsi_path != "none")
   {
-   MDFN_PrintError(_("BIOS ROM file is incorrect size.\n"));
-   return(0);
+   FileStream FXSCSIFile(fxscsi_path.c_str(), FileStream::MODE_READ);
+   uint32 FXSCSI_Map_Addresses[1] = { 0x80780000 };
+
+   FXSCSIROM = PCFX_V810.SetFastMap(FXSCSI_Map_Addresses, 0x0080000, 1, _("FX-SCSI ROM"));
+
+   FXSCSIFile.read(FXSCSIROM, 1024 * 512);
   }
-
-  uint32 FXSCSI_Map_Addresses[1] = { 0x80780000 };
-
-  if(!(FXSCSIROM = PCFX_V810.SetFastMap(FXSCSI_Map_Addresses, 0x0080000, 1, _("FX-SCSI ROM"))))
-  {
-   return(0);
-  }
-
-  memcpy(FXSCSIROM, FXSCSIFile.Data(), 1024 * 512);
-
-  FXSCSIFile.Close();
  }
 
  #ifdef WANT_DEBUGGER
@@ -593,14 +644,7 @@ static bool LoadCommon(std::vector<CDIF *> *CDInterfaces)
  if(WantHuC6273)
   HuC6273_Init();
 
- if(!KING_Init())
- {
-  MDFN_free(BIOSROM);
-  MDFN_free(RAM);
-  BIOSROM = NULL;
-  RAM = NULL;
-  return(0);
- }
+ KING_Init();
 
  CD_TrayOpen = false;
  CD_SelectedDisc = 0;
@@ -624,7 +668,7 @@ static bool LoadCommon(std::vector<CDIF *> *CDInterfaces)
     uint32 sectors;
 
     trio_snprintf(tmpn, 256, "track%d-%d-%d", disc, track, toc.tracks[track].lba);
-    trio_snprintf(tmpln, 256, "CD - Disc %d/%d - Track %d/%d", disc + 1, CDInterfaces->size(), track, toc.last_track - toc.first_track + 1);
+    trio_snprintf(tmpln, 256, "CD - Disc %d/%d - Track %d/%d", disc + 1, (int)CDInterfaces->size(), track, toc.last_track - toc.first_track + 1);
 
     sectors = toc.tracks[track + 1].lba - toc.tracks[track].lba;
     ASpace_Add(PCFXDBG_GetAddressSpaceBytes, PCFXDBG_PutAddressSpaceBytes, tmpn, tmpln, 0, sectors * 2048);
@@ -648,6 +692,9 @@ static bool LoadCommon(std::vector<CDIF *> *CDInterfaces)
  MDFNMP_AddRAM(2048 * 1024, 0x00000000, RAM);
 
 
+ BackupSignalDirty = false;
+ BackupSaveDelay = 0;
+
  BRAMDisabled = MDFN_GetSettingB("pcfx.disable_bram");
 
  if(BRAMDisabled)
@@ -724,10 +771,6 @@ static bool LoadCommon(std::vector<CDIF *> *CDInterfaces)
 
  PCFX_V810.SetIOReadHandlers(port_rbyte, port_rhword, NULL);
  PCFX_V810.SetIOWriteHandlers(port_wbyte, port_whword, NULL);
-
-
-
- return(1);
 }
 
 static void DoMD5CDVoodoo(std::vector<CDIF *> *CDInterfaces)
@@ -881,24 +924,29 @@ static bool TestMagicCD(std::vector<CDIF *> *CDInterfaces)
  return(FALSE);
 }
 
-static int LoadCD(std::vector<CDIF *> *CDInterfaces)
+static void LoadCD(std::vector<CDIF *> *CDInterfaces)
 {
- EmuFlags = 0;
+ try
+ {
+  EmuFlags = 0;
 
- cdifs = CDInterfaces;
+  cdifs = CDInterfaces;
 
- DoMD5CDVoodoo(CDInterfaces);
+  DoMD5CDVoodoo(CDInterfaces);
 
- if(!LoadCommon(CDInterfaces))
-  return(0);
+  LoadCommon(CDInterfaces);
 
- MDFN_printf(_("Emulated CD-ROM drive speed: %ux\n"), (unsigned int)MDFN_GetSettingUI("pcfx.cdspeed"));
+  MDFN_printf(_("Emulated CD-ROM drive speed: %ux\n"), (unsigned int)MDFN_GetSettingUI("pcfx.cdspeed"));
 
- MDFNGameInfo->GameType = GMT_CDROM;
+  MDFNGameInfo->GameType = GMT_CDROM;
 
- PCFX_Power();
-
- return(1);
+  PCFX_Power();
+ }
+ catch(...)
+ {
+  Cleanup();
+  throw;
+ }
 }
 
 static void PCFX_CDInsertEject(void)
@@ -944,32 +992,31 @@ static void PCFX_CDSelect(void)
  }
 }
 
-static void CloseGame(void)
+static void SaveBackupMemory(void)
 {
  if(!BRAMDisabled)
  {
-  std::vector<PtrLengthPair> EvilRams;
- 
-  EvilRams.push_back(PtrLengthPair(BackupRAM, 0x8000));
-  EvilRams.push_back(PtrLengthPair(ExBackupRAM, 0x8000));
+  FileStream fp(MDFN_MakeFName(MDFNMKF_SAV, 0, "sav").c_str(), FileStream::MODE_WRITE);
 
-  MDFN_DumpToFile(MDFN_MakeFName(MDFNMKF_SAV, 0, "sav").c_str(), 0, EvilRams);
+  fp.write(BackupRAM, 0x8000);
+  fp.write(ExBackupRAM, 0x8000);
+
+  fp.close();
+ }
+}
+
+static void CloseGame(void)
+{
+ try
+ {
+  SaveBackupMemory();
+ }
+ catch(std::exception &e)
+ {
+  MDFN_PrintError(_("Error saving save-game memory: %s"), e.what());
  }
 
- for(int i = 0; i < 2; i++)
-  if(fx_vdc_chips[i])
-  {
-   delete fx_vdc_chips[i];
-   fx_vdc_chips[i] = NULL;
-  }
-
- RAINBOW_Close();
- KING_Close();
- PCFX_V810.Kill();
-
- // The allocated memory RAM and BIOSROM is free'd in V810_Kill()
- RAM = NULL;
- BIOSROM = NULL;
+ Cleanup();
 }
 
 static void DoSimpleCommand(int cmd)
@@ -1042,6 +1089,9 @@ static int StateAction(StateMem *sm, int load, int data_only)
 
    SCSICD_SetDisc(CD_TrayOpen, (CD_SelectedDisc >= 0 && !CD_TrayOpen) ? (*cdifs)[CD_SelectedDisc] : NULL, true);
   }
+
+  if(!BRAMDisabled)
+   BackupSignalDirty = true;
  }
 
  //printf("0x%08x, %d %d %d %d\n", load, next_pad_ts, next_timer_ts, next_adpcm_ts, next_king_ts);
