@@ -4,7 +4,8 @@
 bool SoundBox_SetSoundRate(uint32 rate);
 int32 SoundBox_Flush(const v810_timestamp_t timestamp, v810_timestamp_t* new_base_timestamp, int16 *SoundBuf, const int32 MaxSoundFrames);
 void SoundBox_Write(uint32 A, uint16 V, const v810_timestamp_t timestamp);
-int SoundBox_Init(bool arg_EmulateBuggyCodec, bool arg_ResetAntiClickEnabled);
+void SoundBox_Init(bool arg_EmulateBuggyCodec, bool arg_ResetAntiClickEnabled);
+void SoundBox_Kill(void);
 
 void SoundBox_Reset(const v810_timestamp_t timestamp);
 
@@ -16,6 +17,6 @@ v810_timestamp_t SoundBox_ADPCMUpdate(const v810_timestamp_t timestamp);
 
 void SoundBox_ResetTS(const v810_timestamp_t ts_base);
 
-#include <blip/Blip_Buffer.h>
-#include <blip/Stereo_Buffer.h>
+#include <mednafen/sound/Blip_Buffer.h>
+#include <mednafen/sound/Stereo_Buffer.h>
 #endif
