@@ -86,6 +86,8 @@ void PS_CPU::SetHalt(bool status)
 
 void PS_CPU::Power(void)
 {
+ assert(sizeof(ICache) == sizeof(ICache_Bulk));
+
  memset(GPR, 0, sizeof(GPR));
  memset(&CP0, 0, sizeof(CP0));
  LO = 0;
@@ -119,7 +121,7 @@ void PS_CPU::Power(void)
 
  BIU = 0;
 
- memset(ScratchRAM.data32, 0, 1024);
+ memset(ScratchRAM.data8, 0, 1024);
 
  // Not quite sure about these poweron/reset values:
  for(unsigned i = 0; i < 1024; i++)
@@ -131,7 +133,6 @@ void PS_CPU::Power(void)
  GTE_Power();
 }
 
-// FIXME: save/restore icache data.
 int PS_CPU::StateAction(StateMem *sm, int load, int data_only)
 {
  SFORMAT StateRegs[] =
@@ -150,6 +151,15 @@ int PS_CPU::StateAction(StateMem *sm, int load, int data_only)
   SFVAR(BACKED_LDValue),
   SFVAR(LDAbsorb),
 
+  SFVAR(next_event_ts),
+  SFVAR(gte_ts_done),
+  SFVAR(muldiv_ts_done),
+
+  SFVAR(BIU),
+  SFARRAY32(ICache_Bulk, 2048),
+
+  SFARRAY32(CP0.Regs, 32),
+
   SFARRAY(ReadAbsorb, 0x20),
   SFVAR(ReadAbsorbDummy),
   SFVAR(ReadAbsorbWhich),
@@ -157,12 +167,6 @@ int PS_CPU::StateAction(StateMem *sm, int load, int data_only)
 
   SFARRAY(ScratchRAM.data8, 1024),
 
-  SFVAR(next_event_ts),
-  SFVAR(gte_ts_done),
-  SFVAR(muldiv_ts_done),
-
-  SFARRAY32(CP0.Regs, 32),
-
   SFEND
  };
  int ret = MDFNSS_StateAction(sm, load, data_only, StateRegs, "CPU");
@@ -436,6 +440,11 @@ pscpu_timestamp_t PS_CPU::RunReal(pscpu_timestamp_t timestamp_in)
  register uint32 LDWhich;
  register uint32 LDValue;
  
+ //printf("%d %d\n", gte_ts_done, muldiv_ts_done);
+
+ gte_ts_done += timestamp;
+ muldiv_ts_done += timestamp;
+
  BACKING_TO_ACTIVE;
 
  do
@@ -453,8 +462,16 @@ pscpu_timestamp_t PS_CPU::RunReal(pscpu_timestamp_t timestamp_in)
    {
     ACTIVE_TO_BACKING;
 
+    // For save states in step mode.
+    gte_ts_done -= timestamp;
+    muldiv_ts_done -= timestamp;
+
     CPUHook(timestamp, PC);
 
+    // For save states in step mode.
+    gte_ts_done += timestamp;
+    muldiv_ts_done += timestamp;
+
     BACKING_TO_ACTIVE;
    }
 
