@@ -141,14 +141,6 @@ static void CPUHandler(const pscpu_timestamp_t timestamp, uint32 PC)
 {
  std::vector<PSX_BPOINT>::iterator bpit;
 
- if(PC == 0xB0 && CPU->GetRegister(PS_CPU::GSREG_GPR + 9, NULL, 0) == 0x3D)
- {
-  putchar(CPU->GetRegister(PS_CPU::GSREG_GPR + 4, NULL, 0));
-  //exit(1);
-  //puts((const char *)&MainRAM[CPU->GetRegister(PS_CPU::GSREG_GPR + 4, NULL, 0) & 0x1FFFFF]);
- }
-
-
  for(bpit = BreakPointsPC.begin(); bpit != BreakPointsPC.end(); bpit++)
  {
   if(PC >= bpit->A[0] && PC <= bpit->A[1])
@@ -340,11 +332,6 @@ static int GfxDecode_PBN = 0;
 
 static void DoGfxDecode(void)
 {
- unsigned tp_w, tp_h;
-
- tp_w = 256;
- tp_h = 256;
-
  if(GfxDecode_Buf)
  {
   for(int sy = 0; sy < GfxDecode_Buf->h; sy++)
@@ -352,7 +339,7 @@ static void DoGfxDecode(void)
    for(int sx = 0; sx < GfxDecode_Buf->w; sx++)
    {
     unsigned fb_x = ((sx % GfxDecode_Buf->w) + ((sy + GfxDecode_Scroll) / GfxDecode_Buf->w * GfxDecode_Buf->w)) & 1023;
-    unsigned fb_y = (((sy + GfxDecode_Scroll) % GfxDecode_Buf->w) + ((((sx % GfxDecode_Buf->w) + ((sy + GfxDecode_Scroll) / GfxDecode_Buf->w * GfxDecode_Buf->w)) / 1024) * 256)) & 511;
+    unsigned fb_y = (((sy + GfxDecode_Scroll) % GfxDecode_Buf->w) + ((((sx % GfxDecode_Buf->w) + ((sy + GfxDecode_Scroll) / GfxDecode_Buf->w * GfxDecode_Buf->w)) / 1024) * GfxDecode_Buf->w)) & 511;
 
     uint16 pixel = GPU->PeekRAM(fb_y * 1024 + fb_x);
 
