@@ -84,7 +84,7 @@ static Channel DMACH[7];
 static pscpu_timestamp_t lastts;
 
 
-static const char *PrettyChannelNames[7] = { "MDEC IN", "MDEC OUT", "GPU", "CDC", "SPU", "PIO", "OTC" };
+// static const char *PrettyChannelNames[7] = { "MDEC IN", "MDEC OUT", "GPU", "CDC", "SPU", "PIO", "OTC" };
 
 void DMA_Init(void)
 {
@@ -241,7 +241,7 @@ static void RecalcHalt(void)
 }
 
 
-static INLINE void ChRW(const unsigned ch, const uint32 CRModeCache, uint32 *V)
+static INLINE void ChRW(const unsigned ch, const uint32 CRModeCache, uint32 *V, int32 *offset)
 {
  unsigned extra_cyc_overhead = 0;
 
@@ -263,7 +263,7 @@ static INLINE void ChRW(const unsigned ch, const uint32 CRModeCache, uint32 *V)
 	  {
 	  }
 	  else
-	   *V = MDEC_DMARead();
+	   *V = MDEC_DMARead(offset);
 	  break;
 
   case CH_GPU:
@@ -427,6 +427,7 @@ static INLINE void RunChannelI(const unsigned ch, const uint32 CRModeCache, int3
   //
   {
    uint32 vtmp;
+   int32 voffs = 0;
 
    if(MDFN_UNLIKELY(DMACH[ch].CurAddr & 0x800000))
    {
@@ -439,10 +440,10 @@ static INLINE void RunChannelI(const unsigned ch, const uint32 CRModeCache, int3
    if(CRModeCache & 0x1)
     vtmp = MainRAM.ReadU32(DMACH[ch].CurAddr & 0x1FFFFC);
 
-   ChRW(ch, CRModeCache, &vtmp);
+   ChRW(ch, CRModeCache, &vtmp, &voffs);
 
    if(!(CRModeCache & 0x1))
-    MainRAM.WriteU32(DMACH[ch].CurAddr & 0x1FFFFC, vtmp);
+    MainRAM.WriteU32((DMACH[ch].CurAddr + (voffs << 2)) & 0x1FFFFC, vtmp);
   }
 
   if(CRModeCache & 0x2)
@@ -711,7 +712,8 @@ void DMA_Write(const pscpu_timestamp_t timestamp, uint32 A, uint32 V)
 
 	    if(!(OldCC & (1 << 24)) && (V & (1 << 24)))
 	    {
-	     //PSX_WARNING("[DMA] Started DMA for channel=%d --- CHCR=0x%08x --- BCR=0x%08x --- scanline=%d", ch, DMACH[ch].ChanControl, DMACH[ch].BlockControl, GPU->GetScanlineNum());
+	     //if(ch == 0 || ch == 1)
+	     // PSX_WARNING("[DMA] Started DMA for channel=%d --- CHCR=0x%08x --- BCR=0x%08x --- scanline=%d", ch, DMACH[ch].ChanControl, DMACH[ch].BlockControl, GPU->GetScanlineNum());
 
 	     DMACH[ch].WordCounter = 0;
 	     DMACH[ch].ClockCounter = 0;
@@ -803,8 +805,6 @@ int DMA_StateAction(StateMem *sm, int load, int data_only)
 
 #undef SFDMACH
 
-  SFVAR(lastts),
-
   SFEND
  };
  int ret = MDFNSS_StateAction(sm, load, data_only, StateRegs, "DMA");
