@@ -35,7 +35,7 @@
 namespace MDFN_IEN_PSX
 {
 
-InputDevice::InputDevice()
+InputDevice::InputDevice() : chair_r(0), chair_g(0), chair_b(0), draw_chair(0), chair_x(-1000), chair_y(-1000)
 {
 }
 
@@ -47,6 +47,12 @@ void InputDevice::Power(void)
 {
 }
 
+int InputDevice::StateAction(StateMem* sm, int load, int data_only, const char* section_name)
+{
+ return(1);
+}
+
+
 void InputDevice::Update(const pscpu_timestamp_t timestamp)
 {
 
@@ -64,7 +70,58 @@ void InputDevice::SetAMCT(bool)
 
 void InputDevice::SetCrosshairsColor(uint32 color)
 {
+ chair_r = (color >> 16) & 0xFF;
+ chair_g = (color >>  8) & 0xFF;
+ chair_b = (color >>  0) & 0xFF;
 
+ draw_chair = (color != (1 << 24));
+}
+
+INLINE void InputDevice::DrawCrosshairs(uint32 *pixels, const MDFN_PixelFormat* const format, const unsigned width, const unsigned pix_clock)
+{
+ if(draw_chair && chair_y >= -8 && chair_y <= 8)
+ {
+  int32 ic;
+  int32 x_start, x_bound;
+
+  if(chair_y == 0)
+   ic = pix_clock / 762925;
+  else
+   ic = 0;
+
+  x_start = std::max<int32>(0, chair_x - ic);
+  x_bound = std::min<int32>(width, chair_x + ic + 1);
+
+  for(int32 x = x_start; x < x_bound; x++)
+  {
+   int r, g, b, a;
+   int nr, ng, nb;
+
+   format->DecodeColor(pixels[x], r, g, b, a);
+
+   nr = (r + chair_r * 3) >> 2;
+   ng = (g + chair_g * 3) >> 2;
+   nb = (b + chair_b * 3) >> 2;
+
+   if((int)((abs(r - nr) - 0x40) & (abs(g - ng) - 0x40) & (abs(b - nb) - 0x40)) < 0)
+   {
+    if((nr | ng | nb) & 0x80)
+    {
+     nr >>= 1;
+     ng >>= 1;
+     nb >>= 1;
+    }
+    else
+    {
+     nr ^= 0x80;
+     ng ^= 0x80;
+     nb ^= 0x80;
+    }
+   }
+
+   pixels[x] = format->MakeColor(nr, ng, nb, a);
+  }
+ }
 }
 
 bool InputDevice::RequireNoFrameskip(void)
@@ -72,7 +129,7 @@ bool InputDevice::RequireNoFrameskip(void)
  return(false);
 }
 
-pscpu_timestamp_t InputDevice::GPULineHook(const pscpu_timestamp_t timestamp, bool vsync, uint32 *pixels, const MDFN_PixelFormat* const format, const unsigned width, const unsigned pix_clock_offset, const unsigned pix_clock)
+pscpu_timestamp_t InputDevice::GPULineHook(const pscpu_timestamp_t timestamp, bool vsync, uint32 *pixels, const MDFN_PixelFormat* const format, const unsigned width, const unsigned pix_clock_offset, const unsigned pix_clock, const unsigned pix_clock_divider)
 {
  return(PSX_EVENT_MAXTS);
 }
@@ -791,6 +848,74 @@ void FrontIO::SaveMemcard(unsigned int which, const char *path)
  }
 }
 
+int FrontIO::StateAction(StateMem* sm, int load, int data_only)
+{
+ SFORMAT StateRegs[] =
+ {
+  SFVAR(ClockDivider),
+
+  SFVAR(ReceivePending),
+  SFVAR(TransmitPending),
+
+  SFVAR(ReceiveInProgress),
+  SFVAR(TransmitInProgress),
+
+  SFVAR(ReceiveBufferAvail),
+
+  SFVAR(ReceiveBuffer),
+  SFVAR(TransmitBuffer),
+
+  SFVAR(ReceiveBitCounter),
+  SFVAR(TransmitBitCounter),
+
+  SFVAR(Mode),
+  SFVAR(Control),
+  SFVAR(Baudrate),
+
+  SFVAR(istatus),
+
+  // FIXME: Step mode save states.
+  SFARRAY32(irq10_pulse_ts, sizeof(irq10_pulse_ts) / sizeof(irq10_pulse_ts[0])),
+  SFARRAY32(dsr_pulse_delay, sizeof(dsr_pulse_delay) / sizeof(dsr_pulse_delay[0])),
+  SFARRAY32(dsr_active_until_ts, sizeof(dsr_active_until_ts) / sizeof(dsr_active_until_ts[0])),
+
+  SFEND
+ };
+
+ int ret = MDFNSS_StateAction(sm, load, data_only, StateRegs, "FIO");
+
+ for(unsigned i = 0; i < 8; i++)
+ {
+  char tmpbuf[32];
+  trio_snprintf(tmpbuf, sizeof(tmpbuf), "FIODEV%u", i);
+
+  ret &= Devices[i]->StateAction(sm, load, data_only, tmpbuf);
+ }
+
+ for(unsigned i = 0; i < 8; i++)
+ {
+  char tmpbuf[32];
+  trio_snprintf(tmpbuf, sizeof(tmpbuf), "FIOMC%u", i);
+
+  ret &= DevicesMC[i]->StateAction(sm, load, data_only, tmpbuf);
+ }
+
+ for(unsigned i = 0; i < 2; i++)
+ {
+  char tmpbuf[32];
+  trio_snprintf(tmpbuf, sizeof(tmpbuf), "FIOTAP%u", i);
+
+  ret &= DevicesTap[i]->StateAction(sm, load, data_only, tmpbuf);
+ }
+
+ if(load)
+ {
+  IRQ_Assert(IRQ_SIO, istatus);
+ }
+
+ return(ret);
+}
+
 bool FrontIO::RequireNoFrameskip(void)
 {
  for(unsigned i = 0; i < 8; i++)
@@ -800,13 +925,13 @@ bool FrontIO::RequireNoFrameskip(void)
  return(false);
 }
 
-void FrontIO::GPULineHook(const pscpu_timestamp_t timestamp, const pscpu_timestamp_t line_timestamp, bool vsync, uint32 *pixels, const MDFN_PixelFormat* const format, const unsigned width, const unsigned pix_clock_offset, const unsigned pix_clock)
+void FrontIO::GPULineHook(const pscpu_timestamp_t timestamp, const pscpu_timestamp_t line_timestamp, bool vsync, uint32 *pixels, const MDFN_PixelFormat* const format, const unsigned width, const unsigned pix_clock_offset, const unsigned pix_clock, const unsigned pix_clock_divider)
 {
  Update(timestamp);
 
  for(unsigned i = 0; i < 8; i++)
  {
-  pscpu_timestamp_t plts = Devices[i]->GPULineHook(line_timestamp, vsync, pixels, format, width, pix_clock_offset, pix_clock);
+  pscpu_timestamp_t plts = Devices[i]->GPULineHook(line_timestamp, vsync, pixels, format, width, pix_clock_offset, pix_clock, pix_clock_divider);
 
   if(i < 2)
   {
@@ -821,6 +946,17 @@ void FrontIO::GPULineHook(const pscpu_timestamp_t timestamp, const pscpu_timesta
   }
  }
 
+ //
+ // Draw crosshairs in a separate pass so the crosshairs won't mess up the color evaluation of later lightun GPULineHook()s.
+ //
+ if(pixels && pix_clock)
+ {
+  for(unsigned i = 0; i < 8; i++)
+  {
+   Devices[i]->DrawCrosshairs(pixels, format, width, pix_clock);
+  }
+ }
+
  PSX_SetEventNT(PSX_EVENT_FIO, CalcNextEventTS(timestamp, 0x10000000));
 }
 
