@@ -48,8 +48,6 @@ namespace MDFN_IEN_PSX
 {
 #endif
 
-// FIXME: Add alternate structure definitions for big-endian platforms.
-
 typedef struct
 {
  int16 MX[3][3];
@@ -58,10 +56,17 @@ typedef struct
 
 typedef struct
 {
- uint8 R;
- uint8 G;
- uint8 B;
- uint8 CD;
+ union
+ {
+  struct
+  {
+   uint8 R;
+   uint8 G;
+   uint8 B;
+   uint8 CD;
+  };
+  uint8 Raw8[4];
+ };
 } gtergb;
 
 typedef struct
@@ -89,7 +94,9 @@ static uint32 FLAGS;	// Temporary for instruction execution, copied into CR[31]
 typedef union
 {
  gtematrix All[4];
- int32 Raw[4][5];
+ int32 Raw[4][5];	// Don't read from this(Raw[][]), only write(and when writing, if running on a big-endian platform, swap the upper 16-bits with the lower 16-bits)
+ int16 Raw16[4][10];
+
  struct
  {
   gtematrix Rot;
@@ -226,6 +233,66 @@ void GTE_Power(void)
  Reg23 = 0;
 }
 
+// TODO: Don't save redundant state, regarding CR cache variables
+int GTE_StateAction(StateMem *sm, int load, int data_only)
+{
+ SFORMAT StateRegs[] =
+ {
+  SFARRAY32(CR, 32),
+  SFVAR(FLAGS),
+
+  SFARRAY16(&Matrices.Raw16[0][0], 4 * 10),
+
+  SFARRAY32(&CRVectors.All[0][0], 4 * 4),
+
+  SFVAR(OFX),
+  SFVAR(OFY),
+  SFVAR(H),
+  SFVAR(DQA),
+  SFVAR(DQB),
+
+  SFVAR(ZSF3),
+  SFVAR(ZSF4),
+  SFARRAY16(&Vectors[0][0], 3 * 4),
+
+  SFARRAY(RGB.Raw8, 4),
+  SFVAR(OTZ),
+  SFARRAY16(IR, 4),
+
+  SFVAR(XY_FIFO[0].X),
+  SFVAR(XY_FIFO[0].Y),
+  SFVAR(XY_FIFO[1].X),
+  SFVAR(XY_FIFO[1].Y),
+  SFVAR(XY_FIFO[2].X),
+  SFVAR(XY_FIFO[2].Y),
+  SFVAR(XY_FIFO[3].X),
+  SFVAR(XY_FIFO[3].Y),
+
+  SFARRAY16(Z_FIFO, 4),
+
+  SFARRAY(RGB_FIFO[0].Raw8, 4),
+  SFARRAY(RGB_FIFO[1].Raw8, 4),
+  SFARRAY(RGB_FIFO[2].Raw8, 4),
+
+  SFARRAY32(MAC, 4),
+
+  SFVAR(LZCS),
+  SFVAR(LZCR),
+  SFVAR(Reg23),
+
+  SFEND
+ };
+ int ret = MDFNSS_StateAction(sm, load, data_only, StateRegs, "GTE");
+
+ if(load)
+ {
+
+ }
+
+ return(ret);
+}
+
+
 void GTE_WriteCR(unsigned int which, uint32 value)
 {
  static const uint32 mask_table[32] = {
@@ -256,7 +323,13 @@ void GTE_WriteCR(unsigned int which, uint32 value)
   if(which >= 5)
    CRVectors.All[we][which - 5] = value;
   else
+  {
+   #ifdef MSB_FIRST
+   Matrices.Raw[we][which] = (value << 16) | (value >> 16);
+   #else
    Matrices.Raw[we][which] = value;
+   #endif
+  }
   return;
  }
 
@@ -932,15 +1005,13 @@ INLINE void MultiplyMatrixByVector_PT(const gtematrix *matrix, const int16 *v, c
 }
 
 
-#define VAR_UNUSED __attribute__((unused))
-
 #define DECODE_FIELDS							\
- const uint32 sf VAR_UNUSED = (instr & (1 << 19)) ? 12 : 0;		\
- const uint32 mx VAR_UNUSED = (instr >> 17) & 0x3;			\
+ const uint32 sf MDFN_NOWARN_UNUSED = (instr & (1 << 19)) ? 12 : 0;		\
+ const uint32 mx MDFN_NOWARN_UNUSED = (instr >> 17) & 0x3;			\
  const uint32 v_i = (instr >> 15) & 0x3;				\
- const int32* cv VAR_UNUSED = CRVectors.All[(instr >> 13) & 0x3];	\
- const int lm VAR_UNUSED = (instr >> 10) & 1;			\
- int16 v[3];							\
+ const int32* cv MDFN_NOWARN_UNUSED = CRVectors.All[(instr >> 13) & 0x3];	\
+ const int lm MDFN_NOWARN_UNUSED = (instr >> 10) & 1;			\
+ int16 v[3] MDFN_NOWARN_UNUSED;					\
  if(v_i == 3)							\
  {								\
   v[0] = IR1;							\
@@ -953,7 +1024,7 @@ INLINE void MultiplyMatrixByVector_PT(const gtematrix *matrix, const int16 *v, c
   v[1] = Vectors[v_i][1];					\
   v[2] = Vectors[v_i][2];					\
  }
-								
+
 
 int32 SQR(uint32 instr)
 {
