@@ -7,7 +7,7 @@ namespace MDFN_IEN_PSX
 
 enum
 {
- IRQ_VSYNC = 	0,
+ IRQ_VBLANK = 	0,
  IRQ_GPU =	        1,
  IRQ_CD =		2,
  IRQ_DMA =		3,	// Probably
@@ -36,6 +36,7 @@ enum
 uint32 IRQ_GetRegister(unsigned int which, char *special, const uint32 special_len);
 void IRQ_SetRegister(unsigned int which, uint32 value);
 
+int IRQ_StateAction(StateMem *sm, int load, int data_only);
 };
 
 
