@@ -22,12 +22,12 @@
 #include "sio.h"
 #include "cdc.h"
 #include "spu.h"
-#include "../mempatcher.h"
-#include "../PSFLoader.h"
-#include "../player.h"
-#include "../cputest/cputest.h"
+#include <mednafen/mempatcher.h>
+#include <mednafen/PSFLoader.h>
+#include <mednafen/player.h>
 
 #include <stdarg.h>
+#include <ctype.h>
 
 extern MDFNGI EmulatedPSX;
 
@@ -133,12 +133,6 @@ enum
  REGION_EU = 2,
 };
 
-#if 0
-static uint32 PortReadCounter[0x4000] = { 0 };	// Debugging(performance)
-static uint32 ReadCounter = 0;
-static uint32 WriteCounter = 0;
-#endif
-
 static PSF1Loader *psf_loader = NULL;
 static std::vector<CDIF*> *cdifs = NULL;
 static std::vector<const char *> cdifs_scex_ids;
@@ -497,16 +491,6 @@ template<typename T, bool IsWrite, bool Access24> static INLINE void MemRW(pscpu
 
  if(A >= 0x1F801000 && A <= 0x1F802FFF)
  {
-#if 0
-  if(!IsWrite)
-  {
-   ReadCounter++;
-   PortReadCounter[A & 0x3FFF]++;
-  }
-  else
-   WriteCounter++;
-#endif
-
   //if(IsWrite)
   // printf("HW Write%d: %08x %08x\n", (unsigned int)(sizeof(T)*8), (unsigned int)A, (unsigned int)V);
   //else
@@ -533,7 +517,8 @@ template<typename T, bool IsWrite, bool Access24> static INLINE void MemRW(pscpu
      if(timestamp >= events[PSX_EVENT__SYNFIRST].next->event_time)
       PSX_EventHandler(timestamp);
 
-     V = SPU->Read(timestamp, A) | (SPU->Read(timestamp, A | 2) << 16);
+     V = SPU->Read(timestamp, A);
+     V |= SPU->Read(timestamp, A | 2) << 16;
     }
    }
    else
@@ -1013,9 +998,9 @@ static void PSX_Power(void)
 }
 
 
-void PSX_GPULineHook(const pscpu_timestamp_t timestamp, const pscpu_timestamp_t line_timestamp, bool vsync, uint32 *pixels, const MDFN_PixelFormat* const format, const unsigned width, const unsigned pix_clock_offset, const unsigned pix_clock)
+void PSX_GPULineHook(const pscpu_timestamp_t timestamp, const pscpu_timestamp_t line_timestamp, bool vsync, uint32 *pixels, const MDFN_PixelFormat* const format, const unsigned width, const unsigned pix_clock_offset, const unsigned pix_clock, const unsigned pix_clock_divider)
 {
- FIO->GPULineHook(timestamp, line_timestamp, vsync, pixels, format, width, pix_clock_offset, pix_clock);
+ FIO->GPULineHook(timestamp, line_timestamp, vsync, pixels, format, width, pix_clock_offset, pix_clock, pix_clock_divider);
 }
 
 }
@@ -1072,7 +1057,7 @@ static void Emulate(EmulateSpecStruct *espec)
  {
   if(!espec->skip)
   {
-   espec->LineWidths[0].w = ~0;
+   espec->LineWidths[0] = ~0;
    Player_Draw(espec->surface, &espec->DisplayRect, 0, espec->SoundBuf, espec->SoundBufSize);
   }
  }
@@ -1111,23 +1096,9 @@ static void Emulate(EmulateSpecStruct *espec)
    }
   }
  }
-
- #if 0
- printf("read=%6d, write=%6d\n", ReadCounter, WriteCounter);
- ReadCounter = 0;
- WriteCounter = 0;
- printf("HW Port reads for this frame:\n");
- for(unsigned i = 0; i < 0x4000; i++)
- {
-  if(PortReadCounter[i] > 100)
-   printf("0x%08x: %d\n", 0x1f800000 + i, PortReadCounter[i]);
- }
- memset(PortReadCounter, 0, sizeof(PortReadCounter));
- printf("\n");
- #endif
 }
 
-static bool TestMagic(const char *name, MDFNFILE *fp)
+static bool TestMagic(MDFNFILE *fp)
 {
  if(PSFLoader::TestMagic(0x01, fp))
   return(true);
@@ -1463,32 +1434,7 @@ static void InitCommon(std::vector<CDIF *> *CDInterfaces, const bool EmulateMemc
 
  DMA_Init();
 
- if(region == REGION_EU)
- {
-  EmulatedPSX.lcm_width = 2800;
-  EmulatedPSX.lcm_height = (sle + 1 - sls) * 2; //576;
-
-  EmulatedPSX.nominal_width = 377;	// Dunno. :(
-  EmulatedPSX.nominal_height = sle + 1 - sls; //288;
-
-  EmulatedPSX.fb_width = 768;
-  EmulatedPSX.fb_height = 576;
-
-  MDFNGameInfo->VideoSystem = VIDSYS_PAL;
- }
- else
- {
-  EmulatedPSX.lcm_width = 2800;
-  EmulatedPSX.lcm_height = (sle + 1 - sls) * 2; //480;
-
-  EmulatedPSX.nominal_width = 320;
-  EmulatedPSX.nominal_height = sle + 1 - sls; //240;
-
-  EmulatedPSX.fb_width = 768;
-  EmulatedPSX.fb_height = 480;
-
-  MDFNGameInfo->VideoSystem = VIDSYS_NTSC;
- }
+ GPU->FillVideoParams(&EmulatedPSX);
 
  if(cdifs)
  {
@@ -1767,13 +1713,13 @@ void PSF1Loader::HandleEXE(const uint8 *data, uint32 size, bool ignore_pcsp)
 }
 
 static void Cleanup(void);
-static int Load(const char *name, MDFNFILE *fp)
+static int Load(MDFNFILE *fp)
 {
  try
  {
   const bool IsPSF = PSFLoader::TestMagic(0x01, fp);
 
-  if(!TestMagic(name, fp))
+  if(!TestMagic(fp))
    throw MDFN_Error(0, _("File format is unknown to module \"%s\"."), MDFNGameInfo->shortname);
 
 // For testing.
@@ -1785,7 +1731,9 @@ static int Load(const char *name, MDFNFILE *fp)
 
   //CDInterfaces.push_back(CDIF_Open("/home/sarah-projects/psxdev/tests/cd/adpcm.cue", false, false));
   //CDInterfaces.push_back(CDIF_Open("/extra/games/PSX/Tony Hawk's Pro Skater 2 (USA)/Tony Hawk's Pro Skater 2 (USA).cue", false, false));
-  CDInterfaces.push_back(CDIF_Open("/extra/games/PSX/Jumping Flash! (USA)/Jumping Flash! (USA).cue", false, false));
+  //CDInterfaces.push_back(CDIF_Open("/extra/games/PSX/Jumping Flash! (USA)/Jumping Flash! (USA).cue", false, false));
+  //CDInterfaces.push_back(CDIF_Open("/extra/games/PC-FX/Blue Breaker.cue", false, false));
+  CDInterfaces.push_back(CDIF_Open("/dev/cdrom2", true, false));
   InitCommon(&CDInterfaces, !IsPSF, true);
 #else
   InitCommon(NULL, !IsPSF, true);
@@ -1808,15 +1756,14 @@ static int Load(const char *name, MDFNFILE *fp)
  }
  catch(std::exception &e)
  {
-  MDFND_PrintError(e.what());
   Cleanup();
-  return(0);
+  throw;
  }
 
  return(1);
 }
 
-static int LoadCD(std::vector<CDIF *> *CDInterfaces)
+static void LoadCD(std::vector<CDIF *> *CDInterfaces)
 {
  try
  {
@@ -1826,12 +1773,9 @@ static int LoadCD(std::vector<CDIF *> *CDInterfaces)
  }
  catch(std::exception &e)
  {
-  MDFND_PrintError(e.what());
   Cleanup();
-  return(0);
+  throw;
  }
-
- return(1);
 }
 
 static void Cleanup(void)
@@ -1927,15 +1871,24 @@ static void SetInput(int port, const char *type, void *ptr)
 
 static int StateAction(StateMem *sm, int load, int data_only)
 {
- return(0);
+ if(!MDFN_GetSettingB("psx.clobbers_lament"))
+ {
+  return(0);
+ }
+
  SFORMAT StateRegs[] =
  {
   SFVAR(CD_TrayOpen),
   SFVAR(CD_SelectedDisc),
   SFARRAY(MainRAM.data8, 1024 * 2048),
-  //SFARRAY(ScratchRAM.data8, 1024),
   SFARRAY32(SysControl.Regs, 9),
-  //SFARRAY32(next_timestamps, sizeof(next_timestamps) / sizeof(next_timestamps[0])),
+
+  SFVAR(PSX_PRNG.lcgo),
+  SFVAR(PSX_PRNG.x),
+  SFVAR(PSX_PRNG.y),
+  SFVAR(PSX_PRNG.z),
+  SFVAR(PSX_PRNG.c),
+
   SFEND
  };
 
@@ -1944,6 +1897,9 @@ static int StateAction(StateMem *sm, int load, int data_only)
  // Call SetDisc() BEFORE we load CDC state, since SetDisc() has emulation side effects.  We might want to clean this up in the future.
  if(load)
  {
+  if(!cdifs || CD_SelectedDisc >= (int)cdifs->size())
+   CD_SelectedDisc = -1;
+
   CDC->SetDisc(CD_TrayOpen, (CD_SelectedDisc >= 0 && !CD_TrayOpen) ? (*cdifs)[CD_SelectedDisc] : NULL,
 	(CD_SelectedDisc >= 0 && !CD_TrayOpen) ? cdifs_scex_ids[CD_SelectedDisc] : NULL);
  }
@@ -1953,16 +1909,20 @@ static int StateAction(StateMem *sm, int load, int data_only)
  ret &= CPU->StateAction(sm, load, data_only);
  ret &= DMA_StateAction(sm, load, data_only);
  ret &= TIMER_StateAction(sm, load, data_only);
+ ret &= SIO_StateAction(sm, load, data_only);
+
  ret &= CDC->StateAction(sm, load, data_only);
  ret &= MDEC_StateAction(sm, load, data_only);
+ ret &= GPU->StateAction(sm, load, data_only);
  ret &= SPU->StateAction(sm, load, data_only);
- //ret &= FIO->StateAction(sm, load, data_only);
- //ret &= GPU->StateAction(sm, load, data_only);
- ret &= IRQ_StateAction(sm, load, data_only);
+
+ ret &= FIO->StateAction(sm, load, data_only);
+
+ ret &= IRQ_StateAction(sm, load, data_only);	// Do it last.
 
  if(load)
  {
-
+  ForceEventUpdates(0);	// FIXME to work with debugger step mode.
  }
 
  return(ret);
@@ -2290,7 +2250,7 @@ static MDFNSetting PSXSettings[] =
  { "psx.bios_eu", MDFNSF_EMU_STATE, gettext_noop("Path to the Europe SCPH-5502 ROM BIOS"), NULL, MDFNST_STRING, "scph5502.bin" },
 
  { "psx.spu.resamp_quality", MDFNSF_NOFLAGS, gettext_noop("SPU output resampler quality."),
-	gettext_noop("0 is lowest quality and CPU usage, 10 is highest quality and CPU usage.  The resampler that this setting refers to is used for converting from 44.1KHz to the sampling rate of the host audio device Mednafen is using.  Changing Mednafen's output rate, via the \"sound.rate\" setting, to \"44100\" will bypass the resampler, which will decrease CPU usage by Mednafen, and can increase or decrease audio quality, depending on various operating system and hardware factors."), MDFNST_UINT, "5", "0", "10" },
+	gettext_noop("0 is lowest quality and CPU usage, 10 is highest quality and CPU usage.  The resampler that this setting refers to is used for converting from 44.1KHz to the sampling rate of the host audio device Mednafen is using.  Changing Mednafen's output rate, via the \"sound.rate\" setting, to \"44100\" may bypass the resampler, which can decrease CPU usage by Mednafen, and can increase or decrease audio quality, depending on various operating system and hardware factors."), MDFNST_UINT, "5", "0", "10" },
 
 
  { "psx.slstart", MDFNSF_NOFLAGS, gettext_noop("First displayed scanline in NTSC mode."), NULL, MDFNST_INT, "0", "0", "239" },
@@ -2303,6 +2263,8 @@ static MDFNSetting PSXSettings[] =
  { "psx.dbg_level", MDFNSF_NOFLAGS, gettext_noop("Debug printf verbosity level."), NULL, MDFNST_UINT, "0", "0", "4" },
 #endif
 
+ { "psx.clobbers_lament", MDFNSF_NOFLAGS, gettext_noop("Enable experimental save state functionality."), gettext_noop("Save states will destroy your saved game/memory card data if you're careless, and that will make clobber sad.  Poor clobber."), MDFNST_BOOL, "0" },
+
  { NULL },
 };
 
