@@ -15,17 +15,18 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "../mednafen.h"
-#include "../md5.h"
-#include "../general.h"
-#include <../base.hpp>
-#include "../mempatcher.h"
-#include "../PSFLoader.h"
-#include "../player.h"
-#include "../FileStream.h"
-#include "../resampler/resampler.h"
+#include <mednafen/mednafen.h>
+#include <mednafen/md5.h>
+#include <mednafen/general.h>
+#include <mednafen/mempatcher.h>
+#include <mednafen/PSFLoader.h>
+#include <mednafen/player.h>
+#include <mednafen/FileStream.h>
+#include <mednafen/resampler/resampler.h>
 #include <vector>
 
+#include "src/base.hpp"
+
 extern MDFNGI EmulatedSNES;
 
 static void Cleanup(void);
@@ -59,7 +60,7 @@ static double SoundLastRate = 0;
 
 static int32 CycleCounter;
 static MDFN_Surface *tsurf = NULL;
-static MDFN_Rect *tlw = NULL;
+static int32 *tlw = NULL;
 static MDFN_Rect *tdr = NULL;
 static EmulateSpecStruct *es = NULL;
 
@@ -73,58 +74,52 @@ static int16 MouseXLatch[2];
 static int16 MouseYLatch[2];
 static uint8 MouseBLatch[2];
 
+static int16 ScopeXLatch[2];
+static int16 ScopeYLatch[2];
+static uint8 ScopeBLatch[2];
+static uint8 ScopeOSCounter[2];
+static const uint8 ScopeOSCounter_StartVal = 10;
+static const uint8 ScopeOSCounter_TriggerStartThresh = 6;
+static const uint8 ScopeOSCounter_TriggerEndThresh = 2;
+
 static uint8 *CustomColorMap = NULL;
 //static uint32 ColorMap[32768];
 static std::vector<uint32> ColorMap;
 
-static bool LoadCPalette(const char *syspalname, uint8 **ptr, uint32 num_entries)
+static void LoadCPalette(const char *syspalname, uint8 **ptr, uint32 num_entries) MDFN_COLD;
+static void LoadCPalette(const char *syspalname, uint8 **ptr, uint32 num_entries)
 {
  std::string colormap_fn = MDFN_MakeFName(MDFNMKF_PALETTE, 0, syspalname).c_str();
 
  MDFN_printf(_("Loading custom palette from \"%s\"...\n"),  colormap_fn.c_str());
  MDFN_indent(1);
 
- *ptr = NULL;
  try
  {
   FileStream fp(colormap_fn.c_str(), FileStream::MODE_READ);
 
-  if(!(*ptr = (uint8 *)MDFN_malloc(num_entries * 3, _("custom color map"))))
-  {
-   MDFN_indent(-1);
-   return(false);
-  }
+  *ptr = new uint8[num_entries * 3];
 
   fp.read(*ptr, num_entries * 3);
  }
  catch(MDFN_Error &e)
  {
-  if(*ptr)
-  {
-   MDFN_free(*ptr);
-   *ptr = NULL;
-  }
-
   MDFN_printf(_("Error: %s\n"), e.what());
   MDFN_indent(-1);
-  return(e.GetErrno() == ENOENT);        // Return fatal error if it's an error other than the file not being found.
+
+  if(e.GetErrno() != ENOENT)
+   throw;
+
+  return;
  }
  catch(std::exception &e)
  {
-  if(*ptr)
-  {
-   MDFN_free(*ptr);
-   *ptr = NULL;
-  }
-
   MDFN_printf(_("Error: %s\n"), e.what());
   MDFN_indent(-1);
-  return(false);
+  throw;
  }
 
  MDFN_indent(-1);
-
- return(true);
 }
 
 
@@ -161,8 +156,7 @@ static void BlankMissingLines(int ystart, int ybound, const bool interlaced, con
   //printf("Blanked: %d\n", y);
   uint32 *dest_line = tsurf->pixels + (field * tsurf->pitch32) + (y * tsurf->pitch32);
   dest_line[0] = tsurf->MakeColor(0, 0/*rand() & 0xFF*/, 0);
-  tlw[(y << interlaced) + field].x = 0;
-  tlw[(y << interlaced) + field].w = 1;
+  tlw[(y << interlaced) + field] = 1;
  }
 }
 
@@ -202,13 +196,12 @@ void bSNES_v059::Interface::video_scanline(uint16_t *data, unsigned line, unsign
 
  //if(rand() & 1)
  {
-  tlw[(y << interlaced) + field].x = 0;
-  tlw[(y << interlaced) + field].w = width;
+  tlw[(y << interlaced) + field] = width;
  }
 
  if(width == 512 && (source_line[0] & 0x8000))
  {
-  tlw[(y << interlaced) + field].w = 256;
+  tlw[(y << interlaced) + field] = 256;
   for(int x = 0; x < 256; x++)
   {
    uint16 p1 = source_line[(x << 1) | 0] & 0x7FFF;
@@ -308,6 +301,19 @@ void bSNES_v059::Interface::input_poll()
 	MouseYLatch[port] = (int32)MDFN_de32lsb(InputPtr[port] + 4);
 	MouseBLatch[port] = *(uint8 *)(InputPtr[port] + 8);
 	break;
+
+   case bSNES_v059::Input::DeviceSuperScope:
+	{
+	 bool old_ost = (ScopeBLatch[port] & 0x02);
+
+	 ScopeXLatch[port] = (int16)MDFN_de16lsb(InputPtr[port] + 0);
+	 ScopeYLatch[port] = (int16)MDFN_de16lsb(InputPtr[port] + 2);
+	 ScopeBLatch[port] = *(uint8 *)(InputPtr[port] + 4);
+
+	 if(!old_ost && (ScopeBLatch[port] & 0x02))
+	  ScopeOSCounter[port] = ScopeOSCounter_StartVal;
+	}
+	break;
   }
  }
 }
@@ -367,6 +373,48 @@ int16_t bSNES_v059::Interface::input_poll(bool port, unsigned device, unsigned i
 	 }
 	}
 	break;
+
+	case bSNES_v059::Input::DeviceSuperScope:
+	{
+	 assert(port < 2);
+	 switch(id)
+	 {
+	  case bSNES_v059::Input::SuperScopeX:
+		return(ScopeOSCounter[port] ? 1000 : ScopeXLatch[port]);
+		break;
+
+	  case bSNES_v059::Input::SuperScopeY:
+		return(ScopeOSCounter[port] ? 1000 : ScopeYLatch[port]);
+		break;
+
+	  case bSNES_v059::Input::SuperScopeTrigger:
+		{
+		 bool trigo = (bool)(ScopeBLatch[port] & 0x01);
+
+		 if(ScopeOSCounter[port] >= ScopeOSCounter_TriggerEndThresh)
+		  trigo = 1;
+
+		 if(ScopeOSCounter[port] >= ScopeOSCounter_TriggerStartThresh)
+		  trigo = 0;
+
+		 return(trigo);
+		}
+		break;
+
+	  case bSNES_v059::Input::SuperScopeCursor:
+		return((bool)(ScopeBLatch[port] & 0x10));
+		break;
+
+	  case bSNES_v059::Input::SuperScopeTurbo:
+		return((bool)(ScopeBLatch[port] & 0x08));
+		break;
+
+	  case bSNES_v059::Input::SuperScopePause:
+		return((bool)(ScopeBLatch[port] & 0x04));
+		break;
+	 }
+	}
+	break;
  }
 
  return(0);
@@ -502,7 +550,7 @@ static bool SaveLoadMemory(bool load)
 }
 
 
-static bool TestMagic(const char *name, MDFNFILE *fp)
+static bool TestMagic(MDFNFILE *fp)
 {
  if(PSFLoader::TestMagic(0x23, fp))
   return(true);
@@ -531,6 +579,14 @@ static void SetupMisc(bool PAL)
   MDFNGameInfo->nominal_width = MDFN_GetSettingB("snes.correct_aspect") ? (PAL ? 344/*354*/ : 292) : 256;
   MDFNGameInfo->nominal_height = PAL ? 239 : 224;
   MDFNGameInfo->lcm_height = MDFNGameInfo->nominal_height * 2;
+
+  //
+  // SuperScope coordinate translation stuff:
+  //
+  MDFNGameInfo->mouse_scale_x = 256.0 / MDFNGameInfo->nominal_width;
+  MDFNGameInfo->mouse_scale_y = 1.0;
+  MDFNGameInfo->mouse_offs_x = 0.0;
+  MDFNGameInfo->mouse_offs_y = 0.0;
  }
 
  ResampInPos = 0;
@@ -615,7 +671,7 @@ void SNSFLoader::HandleReserved(const uint8 *data, uint32 len)
 
 	 if(((uint64)srd_offset + srd_size) > 0x20000)
 	 {
-	  throw MDFN_Error(0, _("SNSF Reserved Section SRAM block combined offset+size(=%ull) is too large."), (unsigned long long)srd_offset + srd_size);
+	  throw MDFN_Error(0, _("SNSF Reserved Section SRAM block combined offset+size(=%llu) is too large."), (unsigned long long)srd_offset + srd_size);
 	 }
 
 	 printf("SRAM(not implemented yet): %08x %08x\n", srd_offset, srd_size);
@@ -712,7 +768,7 @@ static void Cleanup(void)
 
  if(CustomColorMap)
  {
-  MDFN_free(CustomColorMap);
+  delete[] CustomColorMap;
   CustomColorMap = NULL;
  }
 
@@ -776,7 +832,7 @@ static void CheatMap(bool uics, uint8 bank_lo, uint8 bank_hi, uint16 addr_lo, ui
  }
 }
 
-static int Load(const char *name, MDFNFILE *fp)
+static int Load(MDFNFILE *fp)
 {
  bool PAL = FALSE;
 
@@ -917,17 +973,12 @@ static int Load(const char *name, MDFNFILE *fp)
 
   ColorMap.resize(32768);
 
-  if(!LoadCPalette(NULL, &CustomColorMap, 32768))
-  {
-   Cleanup();
-   return(0);
-  }
+  LoadCPalette(NULL, &CustomColorMap, 32768);
  }
  catch(std::exception &e)
  {
-  MDFND_PrintError(e.what());
   Cleanup();
-  return 0;
+  throw;
  }
 
  return(1);
@@ -949,6 +1000,8 @@ static void Emulate(EmulateSpecStruct *espec)
  tdr = &espec->DisplayRect;
  es = espec;
 
+ for(unsigned i = 0; i < 2; i++)
+  ScopeOSCounter[i] -= (bool)ScopeOSCounter[i];
 
  PrevLine = -1;
 
@@ -960,7 +1013,7 @@ static void Emulate(EmulateSpecStruct *espec)
    tdr->y = 0;
    tdr->w = 1;
    tdr->h = 1;
-   tlw[0].w = 0;	// Mark line widths as valid(ie != ~0; since field == 1 would skip it).
+   tlw[0] = 0;	// Mark line widths as valid(ie != ~0; since field == 1 would skip it).
   }
 
   if(espec->VideoFormatChanged)
@@ -1058,7 +1111,7 @@ static void Emulate(EmulateSpecStruct *espec)
  {
   if(!espec->skip)
   {
-   espec->LineWidths[0].w = ~0;
+   espec->LineWidths[0] = ~0;
    Player_Draw(espec->surface, &espec->DisplayRect, 0, espec->SoundBuf, espec->SoundBufSize);
   }
  }
@@ -1089,6 +1142,44 @@ static void Emulate(EmulateSpecStruct *espec)
 static int StateAction(StateMem *sm, int load, int data_only)
 {
  const uint32 length = bSNES_v059::system.serialize_size();
+ bSNES_v059::Input* inpp = &bSNES_v059::input;
+
+ SFORMAT ExtraStateRegs[] =
+ {
+   SFARRAY16(PadLatch, 8),
+
+   SFARRAY16(MouseXLatch, 2),
+   SFARRAY16(MouseYLatch, 2),
+   SFARRAY(MouseBLatch, 2),
+
+   SFARRAY16(ScopeXLatch, 2),
+   SFARRAY16(ScopeYLatch, 2),
+   SFARRAY(ScopeBLatch, 2),
+   SFARRAY(ScopeOSCounter, 2),
+
+   SFVAR(inpp->latchx),
+   SFVAR(inpp->latchy),
+#define INPP_HELPER(n)				\
+   SFVAR(inpp->port[n].counter0),		\
+   SFVAR(inpp->port[n].counter1),		\
+   SFVAR(inpp->port[n].superscope.x),		\
+   SFVAR(inpp->port[n].superscope.y),		\
+   SFVAR(inpp->port[n].superscope.trigger),	\
+   SFVAR(inpp->port[n].superscope.cursor),	\
+   SFVAR(inpp->port[n].superscope.turbo),	\
+   SFVAR(inpp->port[n].superscope.pause),	\
+   SFVAR(inpp->port[n].superscope.offscreen),	\
+   SFVAR(inpp->port[n].superscope.turbolock),	\
+   SFVAR(inpp->port[n].superscope.triggerlock),	\
+   SFVAR(inpp->port[n].superscope.pauselock)
+
+   INPP_HELPER(0),
+   INPP_HELPER(1),
+
+#undef INPP_HELPER
+
+   SFEND
+ };
 
  if(load)
  {
@@ -1100,10 +1191,7 @@ static int StateAction(StateMem *sm, int load, int data_only)
   SFORMAT StateRegs[] =
   {
    SFARRAYN(ptr, length, "OmniCat"),
-   SFARRAY16(PadLatch, 8),
-   SFARRAY16(MouseXLatch, 2),
-   SFARRAY16(MouseYLatch, 2),
-   SFARRAY(MouseBLatch, 2),
+   { ExtraStateRegs, ~0U, 0, NULL },
    SFEND
   };
 
@@ -1130,7 +1218,9 @@ static int StateAction(StateMem *sm, int load, int data_only)
   if(bSNES_v059::scheduler.sync != bSNES_v059::Scheduler::SyncAll)
    bSNES_v059::system.runtosave();
 
-  serializer state = bSNES_v059::system.serialize();
+  serializer state(length);
+
+  bSNES_v059::system.serialize(state);
 
   assert(state.size() == length);
 
@@ -1139,10 +1229,7 @@ static int StateAction(StateMem *sm, int load, int data_only)
   SFORMAT StateRegs[] =
   {
    SFARRAYN(ptr, length, "OmniCat"),
-   SFARRAY16(PadLatch, 8),
-   SFARRAY16(MouseXLatch, 2),
-   SFARRAY16(MouseYLatch, 2),
-   SFARRAY(MouseBLatch, 2),
+   { ExtraStateRegs, ~0U, 0, NULL },
    SFEND
   };
 
@@ -1263,14 +1350,19 @@ static const InputDeviceInputInfoStruct MouseIDII[0x4] =
  { "right", "Right Button", 1, IDIT_BUTTON, NULL },
 };
 
-#if 0
 static const InputDeviceInputInfoStruct SuperScopeIDII[] =
 {
+ { "x_axis", "X Axis", -1, IDIT_X_AXIS },
+ { "y_axis", "Y Axis", -1, IDIT_Y_AXIS },
 
+ { "trigger", "Trigger", 0, IDIT_BUTTON, NULL  },
+ { "offscreen_shot", "Offscreen Shot(Simulated)", 1, IDIT_BUTTON, NULL  },
+ { "pause", "Pause", 2, IDIT_BUTTON, NULL },
+ { "turbo", "Turbo", 3, IDIT_BUTTON, NULL },
+ { "cursor", "Cursor", 4, IDIT_BUTTON, NULL },
 };
-#endif
 
-static InputDeviceInfoStruct InputDeviceInfoSNESPort[] =
+static InputDeviceInfoStruct InputDeviceInfoSNESPort1[] =
 {
  // None
  {
@@ -1301,7 +1393,41 @@ static InputDeviceInfoStruct InputDeviceInfoSNESPort[] =
   sizeof(MouseIDII) / sizeof(InputDeviceInputInfoStruct),
   MouseIDII,
  },
-#if 0
+};
+
+static InputDeviceInfoStruct InputDeviceInfoSNESPort2[] =
+{
+ // None
+ {
+  "none",
+  "none",
+  NULL,
+  NULL,
+  0,
+  NULL
+ },
+
+ // Gamepad
+ {
+  "gamepad",
+  "Gamepad",
+  NULL,
+  NULL,
+  sizeof(GamepadIDII) / sizeof(InputDeviceInputInfoStruct),
+  GamepadIDII,
+ },
+
+ // Mouse
+ {
+  "mouse",
+  "Mouse",
+  NULL,
+  NULL,
+  sizeof(MouseIDII) / sizeof(InputDeviceInputInfoStruct),
+  MouseIDII,
+ },
+
+ // Super Scope
  {
   "superscope",
   "Super Scope",
@@ -1310,7 +1436,6 @@ static InputDeviceInfoStruct InputDeviceInfoSNESPort[] =
   sizeof(SuperScopeIDII) / sizeof(InputDeviceInputInfoStruct),
   SuperScopeIDII
  },
-#endif
 };
 
 
@@ -1330,8 +1455,8 @@ static InputDeviceInfoStruct InputDeviceInfoTapPort[] =
 
 static const InputPortInfoStruct PortInfo[] =
 {
- { "port1", "Port 1/1A", sizeof(InputDeviceInfoSNESPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoSNESPort, "gamepad" },
- { "port2", "Port 2/2A", sizeof(InputDeviceInfoSNESPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoSNESPort, "gamepad" },
+ { "port1", "Port 1/1A", sizeof(InputDeviceInfoSNESPort1) / sizeof(InputDeviceInfoStruct), InputDeviceInfoSNESPort1, "gamepad" },
+ { "port2", "Port 2/2A", sizeof(InputDeviceInfoSNESPort2) / sizeof(InputDeviceInfoStruct), InputDeviceInfoSNESPort2, "gamepad" },
  { "port3", "Port 2B", sizeof(InputDeviceInfoTapPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoTapPort, "gamepad" },
  { "port4", "Port 2C", sizeof(InputDeviceInfoTapPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoTapPort, "gamepad" },
  { "port5", "Port 2D", sizeof(InputDeviceInfoTapPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoTapPort, "gamepad" },
