@@ -4,24 +4,22 @@
   #define debugvirtual
 #endif
 
-namespace SNES {
+namespace bSNES_v059 {
   struct ChipDebugger {
     virtual bool property(unsigned id, string &name, string &value) = 0;
   };
 
+  #include "cheat/cheat.hpp"
   #include "memory/memory.hpp"
   #include "memory/smemory/smemory.hpp"
 
   #include "ppu/ppu.hpp"
-  #include "ppu/bppu/bppu.hpp"
 
   #include "cpu/cpu.hpp"
   #include "cpu/core/core.hpp"
   #include "cpu/scpu/scpu.hpp"
 
   #include "smp/smp.hpp"
-  #include "smp/core/core.hpp"
-  #include "smp/ssmp/ssmp.hpp"
 
   #include "dsp/dsp.hpp"
   #include "dsp/sdsp/sdsp.hpp"
@@ -29,7 +27,7 @@ namespace SNES {
   #include "system/system.hpp"
   #include "chip/chip.hpp"
   #include "cartridge/cartridge.hpp"
-  #include "cheat/cheat.hpp"
+  //#include "cheat/cheat.hpp"
 
   #include "memory/memory-inline.hpp"
   #include "ppu/ppu-inline.hpp"
