@@ -125,38 +125,8 @@ int smem_read32le(StateMem *st, uint32 *b)
  return(4);
 }
 
-
-static bool ValidateSFStructure(SFORMAT *sf)
-{
- SFORMAT *saved_sf = sf;
-
- while(sf->size || sf->name)
- {
-  SFORMAT *sub_sf = saved_sf;
-  while(sub_sf->size || sub_sf->name)
-  {
-   if(sf != sub_sf)
-   {
-    if(!strncmp(sf->name, sub_sf->name, 32))
-    {
-     printf("Duplicate state variable name: %.32s\n", sf->name);
-    }
-   }
-   sub_sf++;
-  }
-
-  sf++;
- }
- return(1);
-}
-
-
 static bool SubWrite(StateMem *st, SFORMAT *sf, int data_only, const char *name_prefix = NULL)
 {
- // FIXME?  It's kind of slow, and we definitely don't want it on with state rewinding...
- if(!data_only) 
-  ValidateSFStructure(sf);
-
  while(sf->size || sf->name)	// Size can sometimes be zero, so also check for the text name.  These two should both be zero only at the end of a struct.
  {
   if(!sf->size || !sf->v)
@@ -405,7 +375,7 @@ static int ReadStateChunk(StateMem *st, SFORMAT *sf, int size, int data_only)
 
     if(recorded_size != expected_size)
     {
-     printf("Variable in save state wrong size: %s.  Need: %d, got: %d\n", toa + 1, expected_size, recorded_size);
+     printf("Variable in save state wrong size: %s.  Need: %u, got: %u\n", toa + 1, expected_size, recorded_size);
      if(smem_seek(st, recorded_size, SEEK_CUR) < 0)
      {
       puts("Seek error");
@@ -548,7 +518,7 @@ int MDFNSS_StateAction(StateMem *st, int load, int data_only, SFORMAT *sf, const
  return(MDFNSS_StateAction(st, load, data_only, love));
 }
 
-int MDFNSS_SaveSM(StateMem *st, int wantpreview_and_ts, int data_only, const MDFN_Surface *surface, const MDFN_Rect *DisplayRect, const MDFN_Rect *LineWidths)
+int MDFNSS_SaveSM(StateMem *st, int wantpreview_and_ts, int data_only, const MDFN_Surface *surface, const MDFN_Rect *DisplayRect, const int32 *LineWidths)
 {
 	static const char *header_magic = "MDFNSVST";
         uint8 header[32];
@@ -565,14 +535,14 @@ int MDFNSS_SaveSM(StateMem *st, int wantpreview_and_ts, int data_only, const MDF
 	 neowidth = MDFNGameInfo->nominal_width;
 	 neoheight = MDFNGameInfo->nominal_height;
 
-	 if(LineWidths[0].w != ~0)
+	 if(LineWidths[0] != ~0)
  	 {
-	  uint32 first_w = LineWidths[DisplayRect->y].w;
+	  uint32 first_w = LineWidths[DisplayRect->y];
 
 	  for(int y = 0; y < DisplayRect->h; y++)
-	   if(LineWidths[DisplayRect->y + y].w != first_w)
+	   if(LineWidths[DisplayRect->y + y] != first_w)
 	   {
-	    puts("Multires!");
+	    //puts("Multires!");
 	    is_multires = TRUE;
 	   }
 	 }
@@ -611,7 +581,7 @@ int MDFNSS_SaveSM(StateMem *st, int wantpreview_and_ts, int data_only, const MDF
 	 dest_rect.w = neowidth;
 	 dest_rect.h = neoheight;
 
-	 MDFN_ResizeSurface(surface, DisplayRect, (LineWidths[0].w != ~0) ? LineWidths : NULL, dest_surface, &dest_rect);
+	 MDFN_ResizeSurface(surface, DisplayRect, LineWidths, dest_surface, &dest_rect);
 
 	 {
 	  uint32 a, b = 0;
@@ -654,13 +624,12 @@ int MDFNSS_SaveSM(StateMem *st, int wantpreview_and_ts, int data_only, const MDF
 	return(1);
 }
 
-int MDFNSS_Save(const char *fname, const char *suffix, const MDFN_Surface *surface, const MDFN_Rect *DisplayRect, const MDFN_Rect *LineWidths)
+int MDFNSS_Save(const char *fname, const char *suffix, const MDFN_Surface *surface, const MDFN_Rect *DisplayRect, const int32 *LineWidths)
 {
 	StateMem st;
 
 	memset(&st, 0, sizeof(StateMem));
 
-
 	if(!MDFNGameInfo->StateAction)
 	{
 	 MDFN_DispMessage(_("Module \"%s\" doesn't support save states."), MDFNGameInfo->shortname);
@@ -689,17 +658,18 @@ int MDFNSS_Save(const char *fname, const char *suffix, const MDFN_Surface *surfa
 
 	free(st.data);
 
-	SaveStateStatus[CurrentState] = 1;
-	RecentlySavedState = CurrentState;
-
 	if(!fname && !suffix)
+	{
+	 SaveStateStatus[CurrentState] = 1;
+	 RecentlySavedState = CurrentState;
 	 MDFN_DispMessage(_("State %d saved."),CurrentState);
+	}
 
 	return(1);
 }
 
 // Convenience function for movie.cpp
-int MDFNSS_SaveFP(gzFile fp, const MDFN_Surface *surface, const MDFN_Rect *DisplayRect, const MDFN_Rect *LineWidths)
+int MDFNSS_SaveFP(gzFile fp, const MDFN_Surface *surface, const MDFN_Rect *DisplayRect, const int32 *LineWidths)
 {
  StateMem st;
 
@@ -960,7 +930,7 @@ void MDFNI_SelectState(int w)
  MDFND_SetStateStatus(status);
 }  
 
-void MDFNI_SaveState(const char *fname, const char *suffix, const MDFN_Surface *surface, const MDFN_Rect *DisplayRect, const MDFN_Rect *LineWidths)
+void MDFNI_SaveState(const char *fname, const char *suffix, const MDFN_Surface *surface, const MDFN_Rect *DisplayRect, const int32 *LineWidths)
 {
  if(!MDFNGameInfo->StateAction) 
   return;
@@ -1035,10 +1005,8 @@ static int32 bcspos;
 
 void MDFN_StateEvilBegin(void)
 {
- int x;
  std::string srwcompstring;
 
-
  if(!EvilEnabled)
   return;
 
@@ -1057,7 +1025,7 @@ void MDFN_StateEvilBegin(void)
  bcs = (StateMemPacket *)calloc(SRW_NUM, sizeof(StateMemPacket));
  bcspos = 0;
 
- for(x=0;x<SRW_NUM;x++)
+ for(int x = 0; x < SRW_NUM; x++)
  {
   bcs[x].data = NULL;
   bcs[x].compressed_len = 0;
@@ -1073,8 +1041,6 @@ bool MDFN_StateEvilIsRunning(void)
 
 void MDFN_StateEvilEnd(void)
 {
- int x;
-
  if(!EvilEnabled)
   return;
 
@@ -1083,21 +1049,22 @@ void MDFN_StateEvilEnd(void)
   if(MDFNMOV_IsRecording())
    MDFN_StateEvilFlushMovieLove();
 
-  for(x = 0;x < SRW_NUM; x++)
+  for(int x = 0; x < SRW_NUM; x++)
   {
-
    if(bcs[x].data)
     free(bcs[x].data);
    bcs[x].data = NULL;
    bcs[x].compressed_len = 0;
   }
   free(bcs);
+  bcs = NULL;
  }
 }
 
 void MDFN_StateEvilFlushMovieLove(void)
 {
  int bahpos = (bcspos + 1) % SRW_NUM;
+
  for(int x = 0; x < SRW_NUM; x++)
  {
   if(bcs[bahpos].MovieLove.data)
