@@ -43,7 +43,7 @@ typedef uint32_t uint32;
 typedef uint64_t uint64;
 
 
-#if !defined(HAVE_NATIVE64BIT) && SIZEOF_VOID_P >= 8
+#if !defined(HAVE_NATIVE64BIT) && (SIZEOF_VOID_P >= 8 || defined(__x86_64__))
 #define HAVE_NATIVE64BIT 1
 #endif
 
@@ -74,7 +74,7 @@ typedef uint64_t uint64;
   #endif
 
   #define MDFN_ALIGN(n)	__attribute__ ((aligned (n)))
-  #define MDFN_FORMATSTR(a,b,c) __attribute__ ((format (a, b, c)));
+  #define MDFN_FORMATSTR(a,b,c) __attribute__ ((format (a, b, c)))
   #define MDFN_WARN_UNUSED_RESULT __attribute__ ((warn_unused_result))
   #define MDFN_NOWARN_UNUSED __attribute__((unused))
 
