@@ -3,6 +3,7 @@
 
 #include <mednafen/mednafen.h>
 #include <mednafen/hw_cpu/v810/v810_cpu.h>
+#include <mednafen/masmem.h>
 #include <trio/trio.h>
 
 namespace MDFN_IEN_VB
@@ -45,6 +46,8 @@ void VBIRQ_Assert(int source, bool assert);
 
 void VB_ExitLoop(void);
 
+void ForceEventUpdates(const v810_timestamp_t timestamp);
+
 
 uint8 MDFN_FASTCALL MemRead8(v810_timestamp_t &timestamp, uint32 A);
 uint16 MDFN_FASTCALL MemRead16(v810_timestamp_t &timestamp, uint32 A);
