@@ -537,6 +537,9 @@ static INLINE uint16 ReadRegister(int32 &timestamp, uint32 A)
 	     }
 	     break;     // XPSTTS, read-only
 
+  case 0x44: ret = 2;	// VIP version.  2 is a known valid version, while the validity of other numbers is unknown, so we'll just go with 2.
+	     break;
+
   case 0x48:
   case 0x4a:
   case 0x4c:
@@ -735,7 +738,7 @@ uint16 VIP_Read16(int32 &timestamp, uint32 A)
            }
            else
            {
-            ret = le16toh(*(uint16 *)&FB[(A >> 15) & 1][(A >> 16) & 1][A & 0x7FFF]);
+            ret = LoadU16_LE((uint16 *)&FB[(A >> 15) & 1][(A >> 16) & 1][A & 0x7FFF]);
            }
            break;
 
@@ -828,7 +831,7 @@ void VIP_Write16(int32 &timestamp, uint32 A, uint16 V)
            if((A & 0x7FFF) >= 0x6000)
             VIP_MA16W16(CHR_RAM, (A & 0x1FFF) | ((A >> 2) & 0x6000), V);
            else
-            *(uint16 *)&FB[(A >> 15) & 1][(A >> 16) & 1][A & 0x7FFF] = htole16(V);
+            StoreU16_LE((uint16 *)&FB[(A >> 15) & 1][(A >> 16) & 1][A & 0x7FFF], V);
            break;
 
   case 0x2:
