@@ -17,6 +17,7 @@
 
 #include <mednafen/mednafen.h>
 #include "surface.h"
+#include <math.h>
 
 MDFN_PixelFormat::MDFN_PixelFormat()
 {
@@ -50,19 +51,62 @@ MDFN_PixelFormat::MDFN_PixelFormat(const unsigned int p_colorspace, const uint8
  Aprec = 8;
 }
 
-MDFN_Surface::MDFN_Surface(void *const p_pixels, const uint32 p_width, const uint32 p_height, const uint32 p_pitchinpix, const MDFN_PixelFormat &nf)
+MDFN_Surface::MDFN_Surface()
 {
- Init(p_pixels, p_width, p_height, p_pitchinpix, nf);
+ memset(&format, 0, sizeof(format));
+
+ pixels = NULL;
+ pixels8 = NULL;
+ pixels16 = NULL;
+ palette = NULL;
+ pixels_is_external = false;
+ pitchinpix = 0;
+ w = 0;
+ h = 0;
 }
 
-void MDFN_Surface::Init(void *const p_pixels, const uint32 p_width, const uint32 p_height, const uint32 p_pitchinpix, const MDFN_PixelFormat &nf)
+MDFN_Surface::MDFN_Surface(void *const p_pixels, const uint32 p_width, const uint32 p_height, const uint32 p_pitchinpix, const MDFN_PixelFormat &nf, const bool alloc_init_pixels)
+{
+ Init(p_pixels, p_width, p_height, p_pitchinpix, nf, alloc_init_pixels);
+}
+
+#if 0
+void MDFN_Surface::Resize(const uint32 p_width, const uint32 p_height, const uint32 p_pitchinpix)
+{
+ void *ptr = (format.bpp == 16) ? pixels16 : pixels;
+ uint64 old_asize = ((uint64)pitchinpix * (format.bpp >> 3)) * h;
+ uint64 new_asize = ((uint64)p_pitchinpix * (format.bpp >> 3)) * p_height;
+
+ if(!(ptr = realloc(ptr, new_asize)))
+  throw MDFN_Error(ErrnoHolder(errno));
+
+ if(new_asize > old_asize)
+  memset((uint8*)ptr + old_asize, 0x00, new_asize - old_asize);
+
+ if(format.bpp == 16)
+  pixels16 = (uint16*)ptr;
+ else
+  pixels = (uint32*)ptr;
+
+ pitchinpix = p_pitchinpix;
+ w = p_width;
+ h = p_height;
+}
+#endif
+
+void MDFN_Surface::Init(void *const p_pixels, const uint32 p_width, const uint32 p_height, const uint32 p_pitchinpix, const MDFN_PixelFormat &nf, const bool alloc_init_pixels)
 {
  void *rpix = NULL;
- assert(nf.bpp == 16 || nf.bpp == 32);
+ assert(nf.bpp == 8 || nf.bpp == 16 || nf.bpp == 32);
 
  format = nf;
 
- if(nf.bpp == 16)
+ if(nf.bpp == 8)
+ {
+  //assert(!nf.Rshift && !nf.Gshift && !nf.Bshift && !nf.Ashift);
+  //assert(!nf.Rprec && !nf.Gprec && !nf.Bprec && !nf.Aprec);
+ }
+ else if(nf.bpp == 16)
  {
   assert(nf.Rprec && nf.Gprec && nf.Bprec && nf.Aprec);
  }
@@ -78,7 +122,9 @@ void MDFN_Surface::Init(void *const p_pixels, const uint32 p_width, const uint32
  }
 
  pixels16 = NULL;
+ pixels8 = NULL;
  pixels = NULL;
+ palette = NULL;
 
  pixels_is_external = false;
 
@@ -89,12 +135,36 @@ void MDFN_Surface::Init(void *const p_pixels, const uint32 p_width, const uint32
  }
  else
  {
-  if(!(rpix = calloc(1, p_pitchinpix * p_height * (nf.bpp / 8))))
-   throw(1);
+  if(alloc_init_pixels)
+   rpix = calloc(1, p_pitchinpix * p_height * (nf.bpp / 8));
+  else
+   rpix = malloc(p_pitchinpix * p_height * (nf.bpp / 8));
+
+  if(!rpix)
+  {
+   ErrnoHolder ene(errno);
+
+   throw(MDFN_Error(ene.Errno(), "%s", ene.StrError()));
+  }
+ }
+
+ if(nf.bpp == 8)
+ {
+  if(!(palette = (MDFN_PaletteEntry*) calloc(sizeof(MDFN_PaletteEntry), 256)))
+  {
+   ErrnoHolder ene(errno);
+
+   if(!pixels_is_external)
+    free(rpix);
+
+   throw(MDFN_Error(ene.Errno(), "%s", ene.StrError()));
+  }
  }
 
  if(nf.bpp == 16)
   pixels16 = (uint16 *)rpix;
+ else if(nf.bpp == 8)
+  pixels8 = (uint8 *)rpix;
  else
   pixels = (uint32 *)rpix;
 
@@ -109,10 +179,17 @@ void MDFN_Surface::Init(void *const p_pixels, const uint32 p_width, const uint32
 // to boot.
 void MDFN_Surface::SetFormat(const MDFN_PixelFormat &nf, bool convert)
 {
- assert(format.bpp == 16 || format.bpp == 32);
- assert(nf.bpp == 16 || nf.bpp == 32);
+ if(format.bpp != 32 || nf.bpp != 32)
+  printf("%u->%u\n",format.bpp, nf.bpp);
 
- if(nf.bpp == 16)
+ assert(format.bpp == 8 || format.bpp == 16 || format.bpp == 32);
+ assert((nf.bpp == 8 && !convert) || nf.bpp == 16 || nf.bpp == 32);
+
+ if(nf.bpp == 8)
+ {
+
+ }
+ else if(nf.bpp == 16)
  {
 
  }
@@ -121,64 +198,126 @@ void MDFN_Surface::SetFormat(const MDFN_PixelFormat &nf, bool convert)
   assert((nf.Rshift + nf.Gshift + nf.Bshift + nf.Ashift) == 48);
   assert(!((nf.Rshift | nf.Gshift | nf.Bshift | nf.Ashift) & 0x7));
  }
- 
+
  if(nf.bpp != format.bpp)
  {
   void *rpix = calloc(1, pitchinpix * h * (nf.bpp / 8));
   void *oldpix;
 
-  if(nf.bpp == 16)	// 32bpp to 16bpp
+  if(nf.bpp == 8)
+  {
+   assert(!convert);
+
+   pixels8 = (uint8 *)rpix;
+   palette = (MDFN_PaletteEntry*)calloc(sizeof(MDFN_PaletteEntry), 256);
+  }
+  else if(nf.bpp == 16)	// 32bpp or 8bpp to 16bpp
   {
    pixels16 = (uint16 *)rpix;
 
    if(convert)
    {
-    puts("32bpp to 16bpp convert");
-    for(int y = 0; y < h; y++)
+    if(format.bpp == 8)
     {
-     uint32 *srow = &pixels[y * pitchinpix];
-     uint16 *drow = &pixels16[y * pitchinpix];
-
-     for(int x = 0; x < w; x++)
+     puts("8bpp to 16bpp convert");
+     for(int y = 0; y < h; y++)
      {
-      uint32 c = srow[x];
-      int r, g, b, a;
+      uint8 *srow = &pixels8[y * pitchinpix];
+      uint16 *drow = &pixels16[y * pitchinpix];
 
-      DecodeColor(c, r, g, b, a);
-      drow[x] = nf.MakeColor(r, g, b, a);
+      for(int x = 0; x < w; x++)
+      {
+       const MDFN_PaletteEntry &p = palette[srow[x]];
+
+       drow[x] = nf.MakeColor(p.r, p.g, p.b, 0);
+      }
+     }
+    }
+    else
+    {
+     puts("32bpp to 16bpp convert");
+     for(int y = 0; y < h; y++)
+     {
+      uint32 *srow = &pixels[y * pitchinpix];
+      uint16 *drow = &pixels16[y * pitchinpix];
+
+      for(int x = 0; x < w; x++)
+      {
+       uint32 c = srow[x];
+       int r, g, b, a;
+
+       DecodeColor(c, r, g, b, a);
+       drow[x] = nf.MakeColor(r, g, b, a);
+      }
      }
     }
    }
-
-   oldpix = pixels;
-   pixels = NULL;
   }
-  else			// 16bpp to 32bpp
+  else			// 16bpp or 8bpp to 32bpp
   {
    pixels = (uint32 *)rpix;
 
    if(convert)
    {
-    puts("16bpp to 32bpp convert");
-    for(int y = 0; y < h; y++)
+    if(format.bpp == 8)
     {
-     uint16 *srow = &pixels16[y * pitchinpix];
-     uint32 *drow = &pixels[y * pitchinpix];
-
-     for(int x = 0; x < w; x++)
+     puts("8bpp to 32bpp convert");
+     for(int y = 0; y < h; y++)
      {
-      uint32 c = srow[x];
-      int r, g, b, a;
+      uint8 *srow = &pixels8[y * pitchinpix];
+      uint32 *drow = &pixels[y * pitchinpix];
 
-      DecodeColor(c, r, g, b, a);
-      drow[x] = nf.MakeColor(r, g, b, a);
+      for(int x = 0; x < w; x++)
+      {
+       const MDFN_PaletteEntry &p = palette[srow[x]];
+
+       drow[x] = nf.MakeColor(p.r, p.g, p.b, 0);
+      }
+     }
+    }
+    else
+    {
+     puts("16bpp to 32bpp convert");
+     for(int y = 0; y < h; y++)
+     {
+      uint16 *srow = &pixels16[y * pitchinpix];
+      uint32 *drow = &pixels[y * pitchinpix];
+
+      for(int x = 0; x < w; x++)
+      {
+       uint32 c = srow[x];
+       int r, g, b, a;
+
+       DecodeColor(c, r, g, b, a);
+       drow[x] = nf.MakeColor(r, g, b, a);
+      }
      }
     }
    }
-
-   oldpix = pixels16;
-   pixels16 = NULL;
   }
+
+  switch(format.bpp)
+  {
+   default:
+
+   case 32: oldpix = pixels;
+	    pixels = NULL;
+	    break;
+
+   case 16: oldpix = pixels16;
+	    pixels16 = NULL;
+	    break;
+
+   case 8:  oldpix = pixels8;
+	    pixels8 = NULL;
+	    if(palette)
+	    {
+	     free(palette);
+	     palette = NULL;
+	    }
+	    break;
+  }
+
   if(oldpix && !pixels_is_external)
    free(oldpix);
 
@@ -244,7 +383,14 @@ void MDFN_Surface::Fill(uint8 r, uint8 g, uint8 b, uint8 a)
 {
  uint32 color = MakeColor(r, g, b, a);
 
- if(format.bpp == 16)
+ if(format.bpp == 8)
+ {
+  assert(pixels8);
+
+  for(int32 i = 0; i < pitchinpix * h; i++)
+   pixels8[i] = color;
+ }
+ else if(format.bpp == 16)
  {
   assert(pixels16);
 
@@ -268,6 +414,9 @@ MDFN_Surface::~MDFN_Surface()
    free(pixels);
   if(pixels16)
    free(pixels16);
+  if(pixels8)
+   free(pixels8);
+  if(palette)
+   free(palette);
  }
 }
-
