@@ -8,8 +8,7 @@ namespace MDFN_IEN_WSWAN
 
 #ifdef WANT_DEBUGGER
 
-void WSwanDBG_SetCPUCallback(void (*callb)(uint32 PC));
-void WSwanDBG_SetBPCallback(void (*callb)(uint32 PC));
+void WSwanDBG_SetCPUCallback(void (*callb)(uint32 PC, bool bpoint), bool continuous);
 
 void WSwanDBG_FlushBreakPoints(int type);
 void WSwanDBG_AddBreakPoint(int type, unsigned int A1, unsigned int A2, bool logical);
@@ -20,6 +19,7 @@ uint32 WSwanDBG_GetRegister(const std::string &name, std::string *special);
 void WSwanDBG_SetRegister(const std::string &name, uint32 value);
 
 void WSwanDBG_AddBranchTrace(uint16 old_CS, uint16 old_IP, uint16 CS, uint16 IP, bool interrupt);
+void WSwanDBG_EnableBranchTrace(bool enable);
 std::vector<BranchTraceResult> WSwanDBG_GetBranchTrace(void);
 
 void WSwanDBG_CheckBP(int type, uint32 address, unsigned int len);
